from itertools import imap
from rest_framework import serializers
from accounts.business.fields import RecoverTypeField, MemberStatusField
from accounts.models import Token, User, LostKey, Member
from acls.models import Role
from workspaces.models import Workspace


class AuthSerializer(serializers.Serializer):

    email = serializers.EmailField(required=True)
    signature = serializers.CharField(required=True)
    date = serializers.CharField(required=True)


class TokenSerializer(serializers.ModelSerializer):

    class Meta:
        model = Token
        fields = ('token', 'user')


class UserSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(required=True)

    class Meta:
        model = User
        fields = ['id', 'email', 'nickname', 'public_key']

    def restore_fields(self, data, files):
        if self.context.get('view').action != 'create':
            self.fields.get('public_key').read_only = True
        return super(UserSerializer, self).restore_fields(data, files)


class RelatedUserSerializer(UserSerializer):

    read_only = True

    class Meta(UserSerializer.Meta):
        fields = ['id', 'nickname', 'email']


class MemberKeySerializer(serializers.ModelSerializer):

    class Meta(UserSerializer.Meta):
        model = Member
        fields = ['id', 'workspace_key']

    def save_object(self, obj, **kwargs):
        return obj


class UserKeySerializer(serializers.ModelSerializer):

    membership = MemberKeySerializer(many=True)

    class Meta(UserSerializer.Meta):
        model = User
        fields = ['id', 'public_key', 'membership']
        depth = 1

    def to_native(self, obj):
        obj.filtered_members = Member.objects.filter(
            user=self.object, status=MemberStatusField.STATUS_MEMBER)

        self.fields['membership'].source = 'filtered_members'
        return super(UserKeySerializer, self).to_native(obj)

    def validate_membership(self, attrs, source):
        all_modified = attrs.get('membership')

        all_existing = Member.objects.filter(
            user=self.object, status=MemberStatusField.STATUS_MEMBER
        )
        all_existing = list(all_existing)
        all_modified = list(all_modified)
        for exi in all_existing:
            found = False
            for mod in all_modified:
                if exi.id == mod.id:
                    found = True
                if mod.status != MemberStatusField.STATUS_MEMBER:
                    msg = 'Only members with STATUS_MEMBER could be updated'
                    raise serializers.ValidationError(msg)
            if not found:
                msg = 'All memberships keys has to be changed at once'
                raise serializers.ValidationError(msg)

        return attrs


class LostKeyCreateSerializer(serializers.Serializer):
    """
    Custom serializer for the post request
    """

    email = serializers.EmailField(required=True)
    id = serializers.IntegerField(read_only=True)

    class Meta:
        fields = ('id', 'email')

    def validate_email(self, attrs, source):
        """
        Check that the email belongs to a valid user
        set the attribute value to a valid user for latter use
        """
        try:
            attrs[source] = User.objects.get(email=attrs[source])
        except:
            raise serializers.ValidationError("Email not found")

        return attrs

    def save_object(self, obj, **kwargs):
        """
        Before save assign the user to the object
        """
        new_lostkey = LostKey(created_by=obj.get('email'))
        new_lostkey.save()
        obj['id'] = new_lostkey.id


class LostKeySerializer(serializers.ModelSerializer):
    """
    This serializer is used for all the LostKeyViewSet
    supported methods but POST
    """

    created_by = RelatedUserSerializer(read_only=True)

    expires_at = serializers.DateTimeField(read_only=True)
    hash = serializers.CharField(read_only=True)
    memberships = serializers.SerializerMethodField('get_memberships')
    public_key = serializers.CharField(max_length=1024, required=True,
                                       write_only=True)
    recover_type = RecoverTypeField(required=True, write_only=True)

    class Meta:
        model = LostKey
        fields = ('id', 'created_by', 'created_at', 'updated_at',
                  'expires_at', 'memberships', 'public_key', 'recover_type',)

    def restore_object(self, attrs, instance=None):
        """
        Remove attributes that not belongs to the model before assignment
        """
        public_key = attrs.pop('public_key', None)
        recover_type = attrs.pop('recover_type')
        obj = super(LostKeySerializer, self).restore_object(attrs, instance)
        obj.public_key = public_key
        obj.recover_type = recover_type
        return obj

    def get_memberships(self, obj):
        """
        Retrieve all workspaces where user is a member.
        Returns an iterable of objects containing the name and id,
        plus a custom field is_recoverable for each workspace.
        A workspace is recoverable if it is share among any other user
        and its membership status is MemberStatusField.STATUS_MEMBER
        :return :dict {'workspace_id': int, 'workspace_name': str,
        'is_recoverable': bool}
        """
        workspaces = Workspace.objects.all_for_user(obj.created_by)
        return imap(
            lambda workspace:
            {'workspace_id': workspace.id, 'workspace_name': workspace.name,
             'is_recoverable': LostKey.objects.find_workspace_is_recoverable(
                 workspace.id, obj.created_by)},
            workspaces)

    def save_object(self, obj, **kwargs):
        """
        Bind current authorised user to created_by field before saving
        """
        recovery_type = obj.recover_type

        if recovery_type == RecoverTypeField.DISABLE:
            LostKey.objects.disable_lost_key(obj.created_by)
            obj.created_by.public_key = ''
        elif recovery_type == RecoverTypeField.REBUILD:
            LostKey.objects.rebuild_lost_key(obj.created_by)
            obj.created_by.public_key = obj.public_key
        else:
            raise Exception('Unsupported recovery type')
        obj.used = True
        obj.created_by.save()

        super(LostKeySerializer, self).save_object(obj, **kwargs)


class MemberInviteSerializer(serializers.Serializer):

    email = serializers.EmailField(required=True)
    workspace = serializers.PrimaryKeyRelatedField(
        required=True, queryset=Workspace.objects.all())
    send = serializers.BooleanField(required=False, default=True)
    resend = serializers.BooleanField(required=False, default=True)


class MemberResendSerializer(serializers.Serializer):
    resend = serializers.BooleanField(required=False, default=True)


class MemberSerializer(serializers.ModelSerializer):

    email = serializers.SerializerMethodField('get_email')
    nickname = serializers.SerializerMethodField('get_nickname')

    class Meta:
        model = Member
        fields = ('id', 'status', 'email', 'nickname', 'workspace', 'user',
                  'created_at', 'updated_at')

    def get_email(self, obj):
        if obj:
            if (obj.user):
                return obj.user.email
            else:
                return obj.invitation_email

    def get_nickname(self, obj):
        if obj:
            if (obj.user):
                return obj.user.nickname
            else:
                return obj.invitation_email


class RelatedMemberSerializer(MemberSerializer):
    pass


class MemberRoleSerializer(serializers.ModelSerializer):

    created_by = serializers.SerializerMethodField('get_created_by')
    to_type = serializers.SerializerMethodField('get_to_type')
    to_name = serializers.SerializerMethodField('get_to_name')

    def get_created_by(self, obj):
        return RelatedUserSerializer(instance=obj.created_by).data

    def get_to_type(self, obj):
        return obj.type

    def get_to_name(self, obj):
        return obj.get_object().name

    class Meta:
        model = Role
        fields = ('id', 'created_by', 'to_type', 'to_name')


class MemberWorkspaceKeySerializer(serializers.ModelSerializer):

    public_key = serializers.SerializerMethodField('get_public_key')
    status = serializers.IntegerField(read_only=True)

    class Meta:
        model = Member
        fields = ('id', 'public_key', 'workspace_key', 'status')

    def validate_workspace_key(self, attrs, source):
        return attrs

    def get_public_key(self, obj):
        return obj.user.public_key

    def save_object(self, obj, **kwargs):
        obj.status = MemberStatusField.STATUS_MEMBER
        return super(MemberWorkspaceKeySerializer, self).\
            save_object(obj, **kwargs)



from django.db.models.loading import get_model
from django.db.models.manager import Manager

from accounts.business.fields import MemberStatusField
from acls.business.fields import AclLevelField, RoleLevelField
from acls.models import Role
from libs.softdelete.softdelete import SoftDeleteManagerMixin


class WorkspaceManager(SoftDeleteManagerMixin, Manager):

    def all_for_user(self, user):
        workspaces = self.filter(
            acl__user=user,
            acl__level=AclLevelField.LEVEL_READ
        ).distinct()

        return workspaces

    def create_member_with_workspace(self, workspace):
        attrs_needed = ['_user', ]
        if not all(hasattr(workspace, attr) for attr in attrs_needed):
            msg = '_user attribute is required to create related membership'
            raise AttributeError(msg)

        m = get_model('accounts', 'Member')(
            workspace=workspace,
            user=workspace._user,
            status=MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY,
            created_by=workspace._user
        )
        m.save()

        r = Role(
            member=m,
            to_workspace=workspace,
            created_by=workspace._user,
            level=RoleLevelField.LEVEL_WRITE
        )
        r.save()



from django.db.transaction import atomic
from rest_framework.response import Response
from rest_framework.status import HTTP_204_NO_CONTENT
from slugs.models import Slug


class AtomicTransactionMixin(object):
    """
    Class encapsulate atomic transaction for rest_framework ViewSets
    """
    def __init__(self, *args, **kwargs):
        super(AtomicTransactionMixin, self).__init__(*args, **kwargs)
        setattr(self, 'dispatch', atomic(getattr(self, 'dispatch')))


class RetrieveBySlugMixin(object):

    def get_object(self, *args, **kwargs):
        pk = self.kwargs.get('pk')
        numeric = pk.isnumeric()
        if not numeric:
            try:
                o = Slug.objects.get(slug=pk)
                self.kwargs['pk'] = o.object_id
            except:
                pass
        return super(RetrieveBySlugMixin, self).get_object(*args, **kwargs)


class SoftDeleteModelMixin(object):
    """
    Destroy a model instance.
    """
    def destroy(self, request, *args, **kwargs):
        obj = self.get_object()
        self.pre_delete(obj)
        obj.softdelete()
        self.post_delete(obj)
        return Response(status=HTTP_204_NO_CONTENT)



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call
from acls.business.fields import RoleLevelField
from acls.models import Role
from acls.tests.api import create_role_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from workspaces.tests.api import create_workspace_api_call, \
    set_workspace_key_api_call, accept_invitation_api_call, \
    list_workspace_key_api_call, get_workspace_key_api_call


class ApiWorkspaceKeyTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    @unittest.skip("should be fixed asap")
    def test_010_list_workspace_key(self):

        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create third user
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user3token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        user1workspace = create_workspace_api_call(
            user1token, name='u1_workspace').data
        self.assertEqual(Role.objects.all().count(), 1)
        # approve workspace key for ourself (user1)
        response = set_workspace_key_api_call(user1token, 1, 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #invite user 2
        user2member = invite_member_api_call(
            user1token, 'jakub@rclick.cz', user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2role = create_role_api_call(user1token, user2member.get('id'),
                                         to_workspace=user1workspace.get('id'),
                                         level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(
            user2token, user2member.get('id'), user2invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #invite user 3
        user3member = invite_member_api_call(
            user1token, 'marcel@rclick.cz', user1workspace.get('id')).data
        user3invitation = Member.objects.get(
            pk=user3member.get('id')).invitation_hash
        user3role = create_role_api_call(
            user1token, user3member.get('id'),
            to_workspace=user1workspace.get('id'),
            level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(
            user3token, user3member.get('id'), user3invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user1 should see two members without workspace key
        # and one workspace key of himself
        response = list_workspace_key_api_call(user1token)
        self.assertEquals(
            len(response.data),
            3
        )

        # user2 should see no members without workspace key
        # and one workspace key of himself
        response = list_workspace_key_api_call(user2token)
        self.assertEquals(
            len(response.data),
            1
        )

    @unittest.skip("should be fixed asap")
    def test_010_get_workspace_key(self):
        #only member approved member of workspace can see workspace key
        #only manager can see approved member workspace key

        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create third user
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user3token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        user1workspace = create_workspace_api_call(
            user1token, name='u1_workspace').data
        self.assertEqual(Role.objects.all().count(), 1)

        # approve workspace key for ourself (user1)
        response = set_workspace_key_api_call(user1token, 1, 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #invite user 2
        user2member = invite_member_api_call(
            user1token, 'jakub@rclick.cz', user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2role = create_role_api_call(
            user1token, user2member.get('id'),
            to_workspace=user1workspace.get('id'),
            level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(
            user2token, user2member.get('id'), user2invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #invite user 3
        user3member = invite_member_api_call(
            user1token, 'marcel@rclick.cz', user1workspace.get('id')).data
        user3invitation = Member.objects.get(
            pk=user3member.get('id')).invitation_hash
        user3role = create_role_api_call(
            user1token, user3member.get('id'),
            to_workspace=user1workspace.get('id'),
            level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(
            user3token, user3member.get('id'), user3invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user 3 cannot get workspace public key of member 2
        response = get_workspace_key_api_call(
            user3token, user2member.get('id'))
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user 3 cannot get workspace key of user1
        response = get_workspace_key_api_call(user2token, 1)
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user1 should be able to get workspace key of user 2
        response = get_workspace_key_api_call(
            user1token, user2member.get('id'))
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user1 will transfer keys to user2
        response = set_workspace_key_api_call(
            user1token, user2member.get('id'), 'this-is-only-mockup-key')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user 2 should be able to read workspace_key of user 3
        response = get_workspace_key_api_call(
            user2token, user3member.get('id'))
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_020_set_workspace_key(self):
    #transfer key
    #key cannot be empty
    #non_approved member cannot transfer key
    #transfer key cannot be done only once by regular member
    #transfer key can be done repeatedly by workspace manager

    # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create third user
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user3token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        user1workspace = create_workspace_api_call(
            user1token, name='u1_workspace').data
        self.assertEqual(Role.objects.all().count(), 1)
        # approve workspace key for ourself (user1)
        response = set_workspace_key_api_call(user1token, 1, 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #invite user 2
        user2member = invite_member_api_call(
            user1token, 'jakub@rclick.cz', user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2role = create_role_api_call(
            user1token, user2member.get('id'),
            to_workspace=user1workspace.get('id'),
            level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(
            user2token, user2member.get('id'), user2invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #invite user 3
        user3member = invite_member_api_call(
            user1token, 'marcel@rclick.cz', user1workspace.get('id')).data
        user3invitation = Member.objects.get(
            pk=user3member.get('id')).invitation_hash
        user3role = create_role_api_call(
            user1token, user3member.get('id'),
            to_workspace=user1workspace.get('id'),
            level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(
            user3token, user3member.get('id'), user3invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user 3 cannot set workspace key of user2
        response = set_workspace_key_api_call(
            user3token, user2member.get('id'), 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user 3 cannot set workspace key of user1
        response = set_workspace_key_api_call(user2token, 1, 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user1 should be able to set workspace key of user 2
        response = set_workspace_key_api_call(
            user1token, user2member.get('id'), 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user2 should be able to set workspace key of user3
        response = set_workspace_key_api_call(
            user2token, user3member.get('id'), 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user 2 should not be able to rewrite workspace_key of user 3
        response = set_workspace_key_api_call(
            user2token, user3member.get('id'), 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user1 should not be able to rewrite workspace key of user3
        # workspace key can be overriden only in case
        # status=withoutworkspacekey
        response = set_workspace_key_api_call(
            user1token, user3member.get('id'), 'mockup')
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )


def workspacekey_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        ApiWorkspaceKeyTest))
    return suite



from rest_framework.exceptions import APIException as BaseAPIException


class CustomAPIException(BaseAPIException):

    def __init__(self, exception=None, detail=None, status_code=None):
        self.detail = detail or 'Error occured'
        self.status_code = status_code or 400
        if exception:
            if isinstance(exception, self.__class__):
                self.detail = exception.detail
                self.status_code = status_code
                return
            if hasattr(exception, 'messages'):
                self.detail = map(str, exception.messages)
            else:
                self.detail = str(exception)



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_200_OK
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call
from libs.version.context import version_context_manager
from libs.version.manipulator import ACTION_UPDATED
from secrets.business.fields import SecretTypeField
from secrets.tests.api import create_secret_api_call, \
    update_secret_blob_api_call
from vaultier.test.tools import AssertionsMixin, format_response, \
    FileAccessMixin
from vaults.tests.api import create_vault_api_call
from versions.models import Version
from workspaces.tests.api import create_workspace_api_call


class SecretBlobVersionTest(TransactionTestCase, AssertionsMixin,
                            FileAccessMixin):

    def setUp(self):
        version_context_manager.set_enabled(True)

    def create_secret(self):
        # create user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='workspace').data

        #create vault
        vault = create_vault_api_call(user1token,
                                      name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="card_in_vault",
                                    vault=vault.get('id')).data

        #create secret_blob
        secret = create_secret_api_call(
            user1token, name="secret_blob_in_card", card=card.get('id'),
            type=SecretTypeField.SECRET_TYPE_PASSWORD).data

        return user1, user1token, workspace, vault, card, secret

    def upload_blob(self, token, secret, filename, meta='blob_meta'):
        file = self.open_file(filename)
        response = update_secret_blob_api_call(token, secret.get('id'),
                                               blob_data=file,
                                               blob_meta=meta)
        file.close()
        return response

    @unittest.skip("should be fixed asap")
    def test_020_upload_and_versioning(self):
        user1, user1token, workspace, vault, card, secret = list(
            self.create_secret())

        response = self.upload_blob(user1token, secret, 'upload.txt',
                                    'mocked_meta')
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        self.assert_dict(
            response.data,
            {'detail': 'success'}
        )

        response = self.upload_blob(user1token, secret, 'upload.txt',
                                    'mocked_meta_updated')
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        self.assert_dict(
            response.data,
            {'detail': 'success'}
        )

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id')
        )

        # assert user has been stored with version
        self.assertEqual(versions[0].created_by.id, user1.get('id'))

         # 3 versions should be there, create, upload1 and upload2
        self.assertEquals(versions.count(), 3)

        #check version
        versions = Version.objects.filter(
            versioned_type__model='secret',
            versioned_id=secret.get('id'),
            manipulator_id='secret_blob_updated_manipulator',
            action_id=ACTION_UPDATED
        )

         # 1 versions should be there, upload1 and upload2
        self.assertEquals(versions.count(), 2)

        self.assertIsNotNone(versions[1].revert_data)
        self.assertIsNotNone(versions[1].revert_data.get('blob_data'))
        self.assertIsNotNone(versions[1].revert_data.get('blob_meta'))


def secret_blob_version_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        SecretBlobVersionTest))
    return suite



from django.conf import settings
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.deletion import PROTECT
from django.db.models.fields import PositiveIntegerField, CharField
import jsonfield


class VersionManipulatorIdField(models.CharField):
    def __init__(self, *args, **kwargs):
        self.name = "VersionManipulatorIdField",
        self.null = False
        self.max_length = 255
        self.default = None
        super(VersionManipulatorIdField, self).__init__(*args, **kwargs)


class VersionMixin(models.Model):
    _manipulator = None

    action_id = PositiveIntegerField(null=True)

    # action definition
    action_name = CharField(max_length=16, null=True)
    manipulator_id = VersionManipulatorIdField(max_length=255)
    revert_data = jsonfield.JSONField(
        help_text="The serialized form of this version of the model.")

    # serialized object representation
    revert_fields = jsonfield.JSONField(
        help_text="The serialized form of affected model fields.")
    versioned_type = models.ForeignKey(ContentType,
                                       related_name='version_versioned')
    versioned_id = PositiveIntegerField()

    # versioned object relation
    versioned = GenericForeignKey('versioned_type', 'versioned_id')
    versioned_related_type = models.ForeignKey(
        ContentType,
        related_name='version_versioned_related',
        null=True)

    # versioned_related object relation
    versioned_related_id = PositiveIntegerField(null=True)
    versioned_related = GenericForeignKey('versioned_related_type',
                                          'versioned_related_id')
    created_at = models.DateTimeField(auto_now_add=True)

    # timestamp and created_by
    updated_at = models.DateTimeField(auto_now=True)

    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=PROTECT,
                                   null=True)

    class Meta:
        abstract = True

    def get_manipulator(self):
        if not self._manipulator:
            from libs.version.manipulator import factory_manipulator

            self._manipulator = factory_manipulator(self, self.manipulator_id)
        return self._manipulator

    def __unicode__(self):
        return 'Version({}):{}:{}'.format(self.id, self.action_name,
                                          self.action_code)



from django.db import models
from libs.changes.changes import ChangesMixin


class Garage(ChangesMixin, models.Model):
    class Meta:
        db_table = u'garage'

    car1 = models.CharField(max_length=255, default='')
    car2 = models.CharField(max_length=255, default='')



from Crypto.Hash import SHA
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from base64 import b64decode
from datetime import datetime, timedelta
from dateutil import parser as dateparser
from django.utils import timezone
from rest_framework.authentication import BaseAuthentication
from rest_framework.exceptions import AuthenticationFailed
from django.conf import settings
from accounts.models import LostKey, Token, User
from django.utils.timezone import utc


class TokenAuthentication(BaseAuthentication):

    def authenticate(self, request):
        token = request.META.get('HTTP_X_VAULTIER_TOKEN')

        if token is None or token == '' or token == 'null':
            return None, None

        try:
            model = Token.objects.get(token=token)
            """:type : Token"""
            token_renewal_interval = settings.VAULTIER.get(
                'authentication_token_renewal_interval')
            #convert to seconds
            token_renewal_interval *= 60

            td = timezone.now() - model.last_used_at
            if td.total_seconds() > token_renewal_interval:
                model.last_used_at = timezone.now()
                model.save()
        except Token.DoesNotExist:
            raise AuthenticationFailed('Invalid token')

        if not model.user.is_active:
            raise AuthenticationFailed('User inactive or deleted')

        return model.user, token


class HashAuthentication(BaseAuthentication):
    """
    Custom authentication for lostKey api
    """

    def authenticate(self, request, **kwargs):
        """
        Validates a lost_key request by hash, id, and expiration time
        """
        user_hash = request.QUERY_PARAMS.get('hash', None)
        if not user_hash:
            user_hash = request.DATA.get('hash')

        lost_key_id = request.parser_context.get('view').kwargs.get('pk')
        try:
            lost_key = LostKey.objects.get(hash=user_hash,
                                           pk=lost_key_id,
                                           expires_at__gte=datetime.utcnow().
                                           replace(tzinfo=utc),
                                           used=False)

            return lost_key.created_by, lost_key.hash
        except:
            return None


class Authenticator(object):

    @classmethod
    def verify(cls, public_key, content, date, signature):
        """
        :param public_key:
        :param content:
        :param date:
        :param signature:
        :return:
        """
        signature = b64decode(signature)
        key = RSA.importKey(public_key)
        h = SHA.new(content + str(date))
        verifier = PKCS1_v1_5.new(key)
        return verifier.verify(h, signature)

    @classmethod
    def sign(cls, private_key, content, timestamp):
        """
        This is not production code, it is used in tests only

        :param private_key:
        :param content:
        :param timestamp:
        :return:
        """
        key = RSA.importKey(private_key)
        h = SHA.new()
        h.update(content + str(timestamp))
        signer = PKCS1_v1_5.new(key)
        sig = signer.sign(h)
        return sig.encode('base64')

    @classmethod
    def _create_token(cls, user):
        """
        Create new authentication token for user

        :param user:
        :return: Token
        """
        return Token.objects.create(user=user)

    @classmethod
    def authenticate(cls, email=None, date=None, signature=None):

        date_parsed = dateparser.parse(date)
        safe_delta = timedelta(
            seconds=settings.VAULTIER.get('login_safe_timestamp'))
        safe_until = date_parsed + safe_delta
        now = timezone.now()

        if safe_until < now:
            raise Exception('Login timestamp to old')

        #check database for user
        try:
            user = User.objects.get(email=email.lower())
        except User.DoesNotExist:
            return None

        # verify signature
        if cls.verify(user.public_key, email, date, signature):
            return cls._create_token(user)
        return None



from django.db.models.loading import get_model
from libs.changes.changes import SOFT_DELETE, INSERT, UPDATE
from libs.version.condition import RequiredFieldEventCondition
from libs.version.manipulator import register_manipulator_signal, \
    ModelCreatedManipulator, ModelUpdatedManipulator, \
    ModelSoftDeletedManipulator, register_manipulator_class
from ..models import Workspace


class WorkspaceSoftDeletedManipulator(ModelSoftDeletedManipulator):
    """
    deleted workspace related is always workspace
    """
    def determine_versioned_related(self):
        return self.version.versioned


def register_signals():
    register_manipulator_class('workspace_created_manipulator',
                               ModelCreatedManipulator)
    register_manipulator_class('workspace_updated_manipulator',
                               ModelUpdatedManipulator)
    register_manipulator_class('workspace_deleted_manipulator',
                               WorkspaceSoftDeletedManipulator)

    version_cls = get_model('versions', 'Version')

    register_manipulator_signal(
        version_cls=version_cls,
        manipulator_id='workspace_deleted_manipulator',
        condition=RequiredFieldEventCondition(
            required_fields=['deleted_at'],
            required_event=SOFT_DELETE,
            required_sender=Workspace,
        ),
    )

    register_manipulator_signal(
        version_cls=version_cls,
        manipulator_id='workspace_updated_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Workspace,
            required_fields=['name', 'description'],
            required_event=UPDATE,
        ),
    )

    register_manipulator_signal(
        version_cls=version_cls,
        manipulator_id='workspace_created_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Workspace,
            required_fields=None,
            required_event=INSERT,
        ),
    )



from django.db import models
from django.db.models.deletion import PROTECT, CASCADE
from cards.business.managers import CardManager
from cards.business.tree import CardTreeIterator
from libs.softdelete.softdelete import SoftDeleteMixin
from libs.tree.iterator import TreeIterableModelMixin
from libs.changes.changes import ChangesMixin


class Card(SoftDeleteMixin, ChangesMixin, TreeIterableModelMixin,
           models.Model):

    objects = CardManager()

    tree_iterator_class = CardTreeIterator

    name = models.CharField(max_length=255)
    slug = models.CharField(max_length=255, default='')
    description = models.CharField(max_length=1024, blank=True, null=True)
    vault = models.ForeignKey('vaults.Vault', on_delete=CASCADE)
    created_by = models.ForeignKey('accounts.User', on_delete=PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = u'vaultier_card'



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED, HTTP_200_OK,\
    HTTP_400_BAD_REQUEST
from accounts.tests.api import register_api_call, auth_api_call
from cards.tests.api import create_card_api_call
from libs.version.context import version_context_manager
from secrets.business.fields import SecretTypeField
from secrets.tests.api import create_secret_api_call, \
    update_secret_blob_api_call, delete_secret_blob_api_call, \
    retrieve_secret_blob_api_call, list_secrets_blob_api_call, \
    create_secret_blob_api_call
from vaultier.test.tools import AssertionsMixin, format_response, \
    FileAccessMixin
from vaults.tests.api import create_vault_api_call
from workspaces.tests.api import create_workspace_api_call


class SecretBlobTest(TransactionTestCase, AssertionsMixin, FileAccessMixin):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def create_secret(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='workspace').data

        #create vault
        vault = create_vault_api_call(
            user1token, name="vault_in_workspace",
            workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(
            user1token, name="card_in_vault", vault=vault.get('id')).data

        #create secret_blob
        secret = create_secret_api_call(
            user1token, name="secret_blob_in_card", card=card.get('id'),
            type=SecretTypeField.SECRET_TYPE_PASSWORD
        ).data

        return user1token, workspace, vault, card, secret

    def upload_blob(self, token, secret, filename):
        file = self.open_file(filename)
        response = update_secret_blob_api_call(
            token, secret.get('id'), blob_data=file, blob_meta='mocked_meta')
        file.close()
        return response

    @unittest.skip("should be fixed asap")
    def test_010_create(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())

        response = create_secret_blob_api_call(user1token)

        self.assertEqual(
            response.status_code,
            HTTP_405_METHOD_NOT_ALLOWED,
            format_response(response)
        )

    def test_020_delete(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())

        response = delete_secret_blob_api_call(user1token, 1)
        self.assertEqual(
            response.status_code,
            HTTP_405_METHOD_NOT_ALLOWED,
            format_response(response)
        )

    def test_020_update(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())
        response = self.upload_blob(user1token, secret, 'upload.txt')
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        self.assert_dict(
            response.data,
            {'detail': 'success'}
        )

    def test_021_update_max_size_exceeded(self):

        user1token, workspace, vault, card, secret = list(self.create_secret())
        response = self.upload_blob(user1token, secret, 'upload-big.txt')

        self.assertEqual(
            response.status_code,
            HTTP_400_BAD_REQUEST,
            format_response(response)
        )

    def test_030_retrieve(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())
        response = self.upload_blob(user1token, secret, 'upload.txt')
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        self.assert_dict(
            response.data,
            {'detail': 'success'}
        )

        file = self.open_file('upload.txt')
        data = file.read()
        file.close()

        response = retrieve_secret_blob_api_call(user1token, secret.get('id'))

        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        self.assertEqual(
            response.data.get('blob_data'),
            data
        )

        self.assertEqual(
            response.data.get('blob_meta'),
            'mocked_meta'
        )

    @unittest.skip("should be fixed asap")
    def test_040_list(self):
        user1token, workspace, vault, card, secret = list(self.create_secret())
        response = list_secrets_blob_api_call(user1token)
        self.assertEqual(
            response.status_code,
            HTTP_405_METHOD_NOT_ALLOWED,
            format_response(response)
        )


def secret_blob_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(SecretBlobTest))
    return suite



from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from cards.models import Card
from search.serializers import SearchSerializer, SearchResultSerializer
from vaults.models import Vault
from vaultier.business.exceptions import CustomAPIException


class SearchView(APIView):
    permission_classes = (IsAuthenticated,)

    def get(self, request):
        return self.search(request)

    def include_type(self, value, type):
        if value:
            return type in value.lower()
        else:
            return True

    def search(self, request):
        max_results = 5
        serializer = SearchSerializer(data=self.request.QUERY_PARAMS)
        if serializer.is_valid():
            try:
                query = serializer.data.get('query')
                type = serializer.data.get('type')

                vaults = []
                if self.include_type(type, 'vaults'):
                    vaults = Vault.objects.search(request.user, query,
                                                  max_results=max_results)

                cards = []
                if self.include_type(type, 'cards'):
                    cards = Card.objects.search(request.user, query,
                                                max_results=max_results)

                results = SearchResultSerializer(instance={'vaults': vaults,
                                                           'cards': cards},
                                                 context={'request': request})
                return Response(data=results.data)

            except Exception as e:
                raise CustomAPIException(exception=e)

        return Response(status=status.HTTP_400_BAD_REQUEST,
                        data={'detail': serializer.errors})



import urlparse
from django.conf import settings
from vaultier.business.mailer import VaultierMailer


class LostKeyMailer(VaultierMailer):
    """
    Custom VaultierMailer implementation for LostKey case
    """
    subject = '[Vaultier] lost key request'
    _template = 'mailer/lostkey'

    def _build_context(self, **kwargs):
        """
        Adds extra variable to template Context

        :return: Context
        """
        kwargs.update({'url': self._build_url()})
        return super(LostKeyMailer, self)._build_context(**kwargs)

    def send(self, **kwargs):
        """
        Adds lost key recipient

        :param kwargs:
        :return:
        """
        if self.object:
            self.add_to(self.object.created_by.email)
        return super(LostKeyMailer, self).send(**kwargs)

    def _build_url(self):
        """
        Generate frontend url to lost key page

        :return:
        """
        return urlparse.urljoin(
            settings.SITE_URL,
            '#/lostkey/{}/{}/'.format(self.object.id, self.object.hash)
        )



from rest_framework import serializers
from accounts.models import Member
from accounts.serializers import RelatedUserSerializer, RelatedMemberSerializer
from acls.models import Role
from cards.serializers import RelatedCardSerializer
from vaultier.business.fields import RelatedNestedField
from vaults.serializers import RelatedVaultSerializer
from workspaces.serializers import RelatedWorkspaceSerializer


class RoleSerializer(serializers.ModelSerializer):
    created_by = RelatedNestedField(serializer=RelatedUserSerializer,
                                    required=False, read_only=True)
    member = RelatedNestedField(serializer=RelatedMemberSerializer,
                                required=True, queryset=Member.objects.all())
    to_workspace = serializers.PrimaryKeyRelatedField(
        required=False, read_only=False)
    to_vault = serializers.PrimaryKeyRelatedField(
        required=False, read_only=False)
    to_card = serializers.PrimaryKeyRelatedField(
        required=False, read_only=False)

    def validate(self, attrs):
        if not (attrs.get('to_workspace') or attrs.get('to_vault') or
                attrs.get('to_card')):
            msg = 'At least one of to_workspace, to_vault, ' \
                  'to_card has to be set'
            raise serializers.ValidationError(msg)

        return attrs

    def save_object(self, obj, **kwargs):
        try:
            obj.compute_type()
        except:
            msg = 'Role has to be related to_workspace or to_vault or to_card'
            raise serializers.NestedValidationError(msg)
        self.object = Role.objects.create_or_update_role(obj)

    class Meta:
        model = Role
        fields = ('id', 'level', 'member', 'to_workspace', 'to_vault',
                  'to_card', 'created_by', 'created_at', 'updated_at',)


class RoleUpdateSerializer(RoleSerializer):

    def validate(self, attrs):
        return attrs

    def get_fields(self):
        fields = super(RoleUpdateSerializer, self).get_fields()
        for field in fields:
            if not field == 'level':
                fields[field].read_only = True
        return fields


class MemberRolesSerializer(RoleSerializer):
    member = RelatedNestedField(required=True,
                                serializer=RelatedMemberSerializer,
                                queryset=Member.objects.all())
    to_workspace = RelatedNestedField(required=False, read_only=False,
                                      serializer=RelatedWorkspaceSerializer)
    to_vault = RelatedNestedField(required=False, read_only=False,
                                  serializer=RelatedVaultSerializer)
    to_card = RelatedNestedField(required=False, read_only=False,
                                 serializer=RelatedCardSerializer)



# -*- coding: utf-8 -*-
from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from accounts.models import User
from cards.models import Card
from libs.version.context import version_context_manager
from slugs.models import Slug
from vaults.models import Vault
from workspaces.models import Workspace


class SlugTest(TransactionTestCase):
    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_workspace_slug(self):
        u = User(email="jan@rclick.cz")
        u.save()

        # create 1
        # on create new slug should be created
        w1 = Workspace(name="Workspace", created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'workspace')
        self.assertEquals(Slug.objects.all().count(), 1)

        # create 2
        # on create new slug should be created
        w2 = Workspace(name="Workspace", created_by=u)
        w2._user = u
        w2.save()
        self.assertEquals(w2.slug, 'workspace-2')
        self.assertEquals(Slug.objects.all().count(), 2)

        # rename 2
        # on update new slug should be created
        w2.name = 'New Workspace Name'
        w2.save()
        self.assertEquals(w2.slug, 'new-workspace-name')
        self.assertEquals(Slug.objects.all().count(), 3)

        # we preserve permalinks for softdeleted objects
        w2.softdelete()
        self.assertEquals(Slug.objects.all().count(), 3)
        # delete all

        # slugs are removed after delete
        Workspace.objects.include_deleted().delete()
        self.assertEquals(Slug.objects.all().count(), 0)

    def test_020_vault_slug(self):
        u = User(email="jan@rclick.cz")
        u.save()

        w1 = Workspace(name="Workspace", created_by=u)
        w1._user = u
        w1.save()

        prereqs_slug_count = 1

        # create 1
        # on create new slug should be created
        v1 = Vault(name="Vault", created_by=u)
        v1.workspace = w1
        v1._user = u
        v1.save()
        self.assertEquals(v1.slug, 'vault')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 1)

        # create 2
        # on create new slug should be created
        v2 = Vault(name="Vault", created_by=u)
        v2.workspace = w1
        v2._user = u
        v2.save()
        self.assertEquals(v2.slug, 'vault-2')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 2)

        # rename 2
        # on update new slug should be created
        v2.name = 'New Vault Name'
        v2.save()
        self.assertEquals(v2.slug, 'new-vault-name')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 3)

        # we preserve permalinks for softdeleted objects
        v2.softdelete()
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 3)

        # delete all, only workspace slug should stay
        Vault.objects.include_deleted().delete()
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count)

    def test_030_card_slug(self):
        u = User(email="jan@rclick.cz")
        u.save()

        w1 = Workspace(name="Workspace", created_by=u)
        w1._user = u
        w1.save()

        v1 = Vault(name="Vault", created_by=u)
        v1.workspace = w1
        v1._user = u
        v1.save()

        prereqs_slug_count = 2

        # create 1
        # on create new slug should be created
        c1 = Card(name="Card", created_by=u)
        c1.vault = v1
        c1._user = u
        c1.save()
        self.assertEquals(c1.slug, 'card')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 1)

        # create 2
        # on create new slug should be created
        c2 = Card(name="Card", created_by=u)
        c2.vault = v1
        c2._user = u
        c2.save()
        self.assertEquals(c2.slug, 'card-2')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 2)

        # rename 2
        # on update new slug should be created
        c2.name = 'New Card Name'
        c2.save()
        self.assertEquals(c2.slug, 'new-card-name')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 3)

        # we preserve permalinks for softdeleted objects
        c2.softdelete()
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 3)

        # delete all, only workspace slug should stay
        Card.objects.include_deleted().delete()
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count)

    def test_040_slug_features(self):
        u = User(email="jan@rclick.cz")
        u.save()

        # support empty names
        w1 = Workspace(name="", created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'workspace')

        # support empty names
        w1 = Workspace(name="", created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'workspace-2')

        # support numeric names
        w1 = Workspace(name=5, created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'n5')

        # support numeric names
        w1 = Workspace(name=5, created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'n5-2')

        # support numeric names
        w1 = Workspace(name=5.5, created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'n55')

        # support numeric names
        w1 = Workspace(name=-6.6, created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'n66')

        # support numeric names
        w1 = Workspace(name='+5.5.-5', created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'n555')

    def test_050_renaming(self):
        u = User(email="jan@rclick.cz")
        u.save()

        w1 = Workspace(name="original", created_by=u)
        w1._user = u
        w1.save()

        w1.name = 'renamed'
        w1.save()
        self.assertEquals(w1.slug, 'renamed')
        self.assertEquals(Slug.objects.all().count(), 2)

        w1.name = 'original'
        w1.save()
        self.assertEquals(w1.slug, 'original')
        self.assertEquals(Slug.objects.all().count(), 2)

    def test_060_slug_can_be_processed_from_mb_characters(self):
        u = User(email="jan@rclick.cz")
        u.save()

        # create 1
        # on create new slug should be created
        w1 = Workspace(name=u"ÁČĎÉĚÍŇÓŘŠŤÚŮÝŽ", created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, 'acdeeinorstuuyz')
        self.assertEquals(Slug.objects.all().count(), 1)

        prereqs_slug_count = 1

        v1 = Vault(name=u"Vault ÁČĎÉĚÍŇÓŘŠŤÚŮÝŽ", created_by=u)
        v1.workspace = w1
        v1._user = u
        v1.save()
        self.assertEquals(v1.slug, 'vault-acdeeinorstuuyz')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 1)

    def test_070_slug_can_be_processed_from_numeric_values(self):
        u = User(email="jan@rclick.cz")
        u.save()

        # create 1
        # on create new slug should be created
        w1 = Workspace(name=125, created_by=u)
        w1._user = u
        w1.save()
        self.assertEquals(w1.slug, u'n125')
        self.assertEquals(Slug.objects.all().count(), 1)

        prereqs_slug_count = 1

        v1 = Vault(name="Vault 125", created_by=u)
        v1.workspace = w1
        v1._user = u
        v1.save()
        self.assertEquals(v1.slug, 'vault-125')
        self.assertEquals(Slug.objects.all().count(), prereqs_slug_count + 1)


def slug_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(SlugTest))
    return suite



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def create_vault_api_call(token, **kwargs):
    url = reverse('vault-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, kwargs)
    return response


def delete_vault_api_call(token, id):
    url = reverse('vault-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def update_vault_api_call(token, id, **kwargs):
    url = reverse('vault-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url, kwargs)
    return response


def retrieve_vault_api_call(token, id):
    url = reverse('vault-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_vaults_api_call(token, workspace=None):
    url = reverse('vault-list')
    client = VaultierAPIClient()
    client.token(token)

    data = {}
    if workspace:
        data['workspace'] = workspace

    response = client.get(url, **data)
    return response



from django.db.models import CharField


class LowerCaseCharField(CharField):
    """
    Defines a charfield which automatically converts all inputs to
    lowercase and saves.
    """

    def pre_save(self, model_instance, add):
        """
        Converts the string to lowercase before saving.
        """
        current_value = getattr(model_instance, self.attname)
        if current_value:
            setattr(model_instance, self.attname, current_value.lower())
        return getattr(model_instance, self.attname)

    def to_python(self, value):
        value = super(LowerCaseCharField, self).to_python(value)
        if isinstance(value, basestring):
            return value.lower()
        return value



import re
from django.template.defaultfilters import slugify


def unique_slugify(
        value,
        queryset,
        instance=None,
        max_length=255,
        slug_field_name="slug",
        default_slug='slug',
        slug_separator='-'):

    # trim slug before
    try:
        value = value.strip()
    except:
        # cast to str only if value is not a unicode
        value = str(value).strip()

    # disallow empty slugs
    if value == '':
        value = default_slug

    # Sort out the initial slug, limiting its length if necessary.
    slug = slugify(value)

    # disable non numeric slug, not to confuse with pks
    num = unicode(slug).replace('-', '')
    if num.isnumeric():
        slug = 'n' + num

    if max_length:
        slug = slug[:max_length]
    slug = _slug_strip(slug, slug_separator)
    original_slug = slug

    if instance and instance.pk:
        queryset = queryset.exclude(pk=instance.pk)

    # Find a unique slug. If one matches, at '-2' to the end and try again
    # (then '-3', etc).
    next = 2
    while not slug or queryset.filter(**{slug_field_name: slug}):
        slug = original_slug
        end = '%s%s' % (slug_separator, next)
        if max_length and len(slug) + len(end) > max_length:
            slug = slug[:max_length - len(end)]
            slug = _slug_strip(slug, slug_separator)
        slug = '%s%s' % (slug, end)
        next += 1

    return slug


def _slug_strip(value, separator='-'):
    """
    Cleans up a slug by removing slug separator characters that occur at the
    beginning or end of a slug.

    If an alternate separator is used, it will also replace any instances of
    the default '-' separator with the new separator.
    """
    separator = separator or ''
    if separator == '-' or not separator:
        re_sep = '-'
    else:
        re_sep = '(?:-|%s)' % re.escape(separator)
        # Remove multiple instances and if an alternate separator is provided,
    # replace the default '-' separator.
    if separator != re_sep:
        value = re.sub('%s+' % re_sep, separator, value)
        # Remove separator from the beginning and end of the slug.
    if separator:
        if separator != '-':
            re_sep = re.escape(separator)
        value = re.sub(r'^%s+|%s+$' % (re_sep, re_sep), '', value)
    return value



from django.core.urlresolvers import reverse
from vaultier.test.tools import VaultierAPIClient


def create_secret_api_call(token, **kwargs):
    url = reverse('secret-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, kwargs)
    return response


def delete_secret_api_call(token, id):
    url = reverse('secret-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def retrieve_secret_api_call(token, id):
    url = reverse('secret-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_secrets_api_call(token, card=None):
    url = reverse('secret-list')
    client = VaultierAPIClient()
    client.token(token)

    data = {}
    if (card):
        data['card'] = card

    response = client.get(url, **data)
    return response


def update_secret_api_call(token, id, **kwargs):
    url = reverse('secret-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.patch(url, data=kwargs)
    return response


def create_secret_blob_api_call(token, **kwargs):
    url = reverse('secret_blob-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, kwargs)
    return response


def delete_secret_blob_api_call(token, id):
    url = reverse('secret_blob-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def retrieve_secret_blob_api_call(token, id):
    url = reverse('secret_blob-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def update_secret_blob_api_call(token, id, **kwargs):
    url = reverse('secret_blob-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.patch(url, kwargs, format='multipart')
    return response


def list_secrets_blob_api_call(token, **kwargs):
    url = reverse('secret_blob-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url, **kwargs)
    return response



from datetime import timedelta
from django.conf import settings
from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_200_OK
from accounts.business.fields import MemberStatusField
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call
from acls.business.fields import RoleLevelField
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response, AssertionsMixin
from vaultier.test.tools.role.api import create_role_api_call
from vaultier.test.tools.workspace.api import create_workspace_api_call, \
    retrieve_workspace_api_call
from vaultier.test.tools.workspacekey.api import set_workspace_key_api_call
from django.utils import timezone
from workspaces.tests.api import accept_invitation_api_call


class ApiInviteTest(TransactionTestCase, AssertionsMixin):
    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_000_invitation(self):
        # create first user
        email = 'tomas@rclick.cz'
        register_api_call(email=email, nickname='tomas').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data

        # user1 tries to invite user2
        response = invite_member_api_call(user1token,
                                          email='jan.misek@rclick.cz',
                                          workspace=workspace1.get('id'),
                                          send=True,
                                          resend=True)
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response))

    def test_010_invitation_should_be_case_insensitive(self):
        # create first user
        user1email = 'tomas@rclick.cz'
        register_api_call(email=user1email, nickname='tomas').data
        user1token = auth_api_call(email=user1email).data.get('token')

        # create second user
        user2email = 'upper_case_user@rclick.cz'
        register_api_call(email=user2email, nickname='UPPER').data
        user2 = auth_api_call(email=user2email).data
        user2token = user2.get('token')
        user2.get('user')

        # create workspace
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data
        set_workspace_key_api_call(user1token, 1, '__KEY__')

        # user1 tries to invite user2 using upper case email
        user2member = invite_member_api_call(user1token,
                                             'UPPER_CASE_USER@rclick.cz',
                                             workspace1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        create_role_api_call(user1token,
                             user2member.get('id'),
                             to_workspace=workspace1.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )
        response = accept_invitation_api_call(user2token,
                                              user2member.get('id'),
                                              user2invitation)

        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response))

        # user 2 can access to the workspace but still
        # it has not set the public key
        response = retrieve_workspace_api_call(user2token,
                                               workspace1.get('id'))
        # print response
        retrieve_workspace_fixture = {
            'id': workspace1.get('id'),
            'perms': {'read': True}
        }
        self.assert_dict(response.data, retrieve_workspace_fixture)
        self.assert_dict(
            response.data.get('membership'),
            {'status': MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY,
             'id': user2member.get('id')}
        )

        response = set_workspace_key_api_call(user1token,
                                              user2member.get('id'), '__KEY__')
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user 2 can access to the workspace but still it has not set
        # the public key
        response = retrieve_workspace_api_call(user2token,
                                               workspace1.get('id'))
        # print response
        retrieve_workspace_fixture = {
            'id': workspace1.get('id'),
            'perms': {'read': True}
        }
        self.assert_dict(response.data, retrieve_workspace_fixture)

        # The user2 has can access its invitation and fulfill every step of
        # the invitation process So the workspace was right assigned
        # to the user no matter the case of the email
        self.assert_dict(response.data.get('membership'),
                         {'status': MemberStatusField.STATUS_MEMBER,
                          'id': user2member.get('id')})

    def test_020_old_invitation_cleaned(self):
        # create first user
        user1email = 'ctirad@rclick.cz'
        register_api_call(email=user1email, nickname='ctirad')
        user1token = auth_api_call(email=user1email).data.get('token')

        # create workspace
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace1').data
        response = set_workspace_key_api_call(user1token, 1, '__KEY__')

        # user1 tries to invite user2 using upper case email
        user2member = invite_member_api_call(user1token, 'user2@rclick.cz',
                                             workspace1.get('id')).data

        member_model = Member.objects.get(pk=user2member.get('id'))
        """:type : Member """
        invitation_lifetime = settings.VAULTIER.get('invitation_lifetime')

        # change date of invitation
        member_model.created_at = timezone.now() - timedelta(
            days=2 * invitation_lifetime)
        member_model.save()

        # invite second user
        user3member = invite_member_api_call(user1token, 'user3@rclick.cz',
                                             workspace1.get('id')).data

        Member.objects.clean_old_invitations()
        # first invitation too old, it should be deleted
        self.assertEqual(
            Member.objects.filter(pk=user2member.get('id')).count(), 0)
        # second invitation should stay
        self.assertEqual(
            Member.objects.filter(pk=user3member.get('id')).count(), 1)


def invitation_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiInviteTest))
    return suite



from rest_framework.mixins import UpdateModelMixin, RetrieveModelMixin, \
    ListModelMixin
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.viewsets import GenericViewSet, ModelViewSet
from rest_framework import status

from accounts.business.authentication import TokenAuthentication
from accounts.business.fields import MemberStatusField
from accounts.models import Member
from libs.version.context import VersionContextAwareApiViewMixin
from vaultier.business.mixins import AtomicTransactionMixin, \
    RetrieveBySlugMixin, SoftDeleteModelMixin
from workspaces.business.permissions import CanManageWorkspace, \
    CanManageWorkspaceKey
from workspaces.models import Workspace
from workspaces.serializers import InvitationSerializer, WorkspaceSerializer, \
    ShortenedWorkspaceKeySerializer, WorkspaceKeySerializer


class InvitationViewSet(AtomicTransactionMixin, UpdateModelMixin,
                        RetrieveModelMixin, GenericViewSet):

    model = Member
    serializer_class = InvitationSerializer
    authentication_classes = (TokenAuthentication, )  # todo: remove this
    permission_classes = (IsAuthenticated,)
    lookup_field = 'invitation_hash'

    def partial_update(self, request, *args, **kwargs):
        return Response(status=status.HTTP_405_METHOD_NOT_ALLOWED,
                        data={'detail': 'Method not allowed'})

    def update(self, request, *args, **kwargs):
        member = self.get_object()
        Member.objects.accept_invitation(member, request.user)
        return Response(status=status.HTTP_200_OK,
                        data=InvitationSerializer(instance=member).data)

    def get_queryset(self):
        return Member.objects.filter(status=MemberStatusField.STATUS_INVITED)


class WorkspaceViewSet(AtomicTransactionMixin, RetrieveBySlugMixin,
                       SoftDeleteModelMixin, VersionContextAwareApiViewMixin,
                       ModelViewSet):
    """
    API endpoint that allows workspaces to be viewed or edited.
    """
    model = Workspace
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated, CanManageWorkspace)
    serializer_class = WorkspaceSerializer

    def get_serializer(self, *args, **kwargs):
        serializer = super(WorkspaceViewSet, self).\
            get_serializer(*args, **kwargs)
        serializer.user = self.request.user
        return serializer

    def pre_save(self, object):
        if object.pk is None:
            object._user = self.request.user
            object.created_by = self.request.user
        return super(WorkspaceViewSet, self).pre_save(object)

    def get_queryset(self):
        if self.action == 'list':
            queryset = Workspace.objects.all_for_user(self.request.user)
        else:
            queryset = Workspace.objects.all()
        return queryset


class WorkspaceKeyViewSet(AtomicTransactionMixin,
                          RetrieveModelMixin,
                          ListModelMixin,
                          UpdateModelMixin,
                          GenericViewSet):
    model = Member
    permission_classes = (IsAuthenticated, CanManageWorkspaceKey)

    def partial_update(self, request, *args, **kwargs):
        return Response(status=status.HTTP_405_METHOD_NOT_ALLOWED,
                        data={'detail': 'Partial update not provided'})

    def get_serializer_class(self):
        if self.action == 'list':
            return ShortenedWorkspaceKeySerializer
        else:
            return WorkspaceKeySerializer

    def get_queryset(self):
        if self.action == 'list':
            return Member.objects.all_to_transfer_keys(self.request.user)
        else:
            return Member.objects.all_for_user(self.request.user)



from rest_framework import status, mixins
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet, GenericViewSet
from libs.version.context import VersionContextAwareApiViewMixin
from secrets.business.permissions import CanManageSecretPermission
from secrets.models import Secret
from secrets.serializers import SecretSerializer, SecretBlobSerializer
from vaultier.business.mixins import AtomicTransactionMixin, \
    SoftDeleteModelMixin


class SecretViewSet(AtomicTransactionMixin, SoftDeleteModelMixin,
                    VersionContextAwareApiViewMixin, ModelViewSet):
    """
    API endpoint that allows secrets to be viewed or edited.
    """
    model = Secret
    serializer_class = SecretSerializer
    permission_classes = (IsAuthenticated, CanManageSecretPermission)
    filter_fields = ('card',)

    def pre_save(self, object):
        if object.pk is None:
            self.check_object_permissions(self.request, object)
            object.created_by = self.request.user
        return super(SecretViewSet, self).pre_save(object)

    def get_queryset(self):
        if self.action == 'list':
            queryset = Secret.objects.all_for_user(self.request.user)
        else:
            queryset = Secret.objects.all()
        return queryset


class SecretBlobViewSet(AtomicTransactionMixin,
                        VersionContextAwareApiViewMixin,
                        mixins.RetrieveModelMixin, mixins.UpdateModelMixin,
                        GenericViewSet):
    """
    API endpoint that allows secret blobs to be viewed or edited.
    """
    model = Secret
    serializer_class = SecretBlobSerializer
    permission_classes = (IsAuthenticated, CanManageSecretPermission)

    def update(self, request, *args, **kwargs):
        response = super(SecretBlobViewSet, self).\
            update(request, *args, **kwargs)

        if response.status_code == status.HTTP_200_OK:
            response.data = {'detail': 'success'}
        return response

    def partial_update(self, request, *args, **kwargs):
        response = super(SecretBlobViewSet, self).\
            partial_update(request, *args, **kwargs)

        if response.status_code == status.HTTP_200_OK:
            response.data = {'detail': 'success'}
        return response

    def get_queryset(self):
        if self.action == 'list':
            queryset = Secret.objects.all_for_user(self.request.user)
        else:
            queryset = Secret.objects.all()
        return queryset



from base import *  # noqa

DEBUG = True
TEMPLATE_DEBUG = DEBUG

# Email address to allow send_mail() method
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

VAULTIER.update({
    # for all users same private key is used
    'dev_shared_key': True,
    # Token is available to copy/paste in user profile box
    'dev_show_token': True,
    # current developer email used for default logging when developing
    'dev_email': '',
    # all emails goes to only this use
    'dev_mail_to': '',
    # 1 hour in milliseconds, used to calculate the expiration date
    # of a lostkey resource
    'lostkey_hash_expiration_time': 3600000,
    # Default email address from which we send emails
    'from_email': 'info@localhost',
    # Max difference between timestamp from server and from front-end
    # (in seconds)
    'login_safe_timestamp': 15,
})

SECRET_KEY = 'DEV_KEY_DEV_KEY_DEV_KEY_DEV_KEY_'



from django.conf.urls import patterns, include, url
from django.contrib import admin
from .views import index, ConfigView, error404

admin.autodiscover()

handler404 = error404

urlpatterns = patterns(
    '',
    url(r'^api/', include('vaultier.urls_api')),
    url(r'^config/config.js$', ConfigView.as_view(), name='config'),

    url(r'^$', index, name='index')
    # Frontend
)



from django.conf import settings
import requests
from django.core.cache import cache
from urlparse import urljoin


class NewsPuller(object):
    """
    NewsPuller service return news list from vaultier.org API if any.
    """
    PREFIX_KEY = '_news'
    ETAG_KEY = '{}-etag'.format(PREFIX_KEY)
    DATA_KEY = '{}-data'.format(PREFIX_KEY)
    _url = None
    _connection_timeout = None
    _cache_timeout = None

    def __init__(self, url, connection_timeout, cache_timeout):
        self._url = url
        self._connection_timeout = connection_timeout
        self._cache_timeout = cache_timeout

    @property
    def etag(self):
        """
        Return etag from cache if any, to be used in request header to
        prevent sending same data

        :return:
        """
        return cache.get(self.ETAG_KEY)

    def _invoke_api(self, count, use_etag=True):
        """
        Invoke vaultier.org website api to get news if any

        :param use_etag:
        :return:
        """
        headers = {'content-type': 'application/json'}
        if use_etag and self.etag:
            headers.update({'If-None-Match': self.etag})

        try:
            url = urljoin(self._url, '?page_size={}'.format(count))
            return requests.get(url, headers=headers,
                                timeout=self._connection_timeout)
        except requests.exceptions.RequestException as e:
            return None

    def _load_from_cache(self):
        """
        Load news from cache if any, otherwise return empty list

        :return: list
        """
        return cache.get(self.DATA_KEY, [])

    def _save(self, data, etag):
        """
        Store acquired data end etag in cache

        :param data:
        :param etag:
        :return:
        """
        cache.set(self.ETAG_KEY, etag, self._cache_timeout)
        cache.set(self.DATA_KEY, data, self._cache_timeout)

    def fetch(self, count=3):
        """
        Fetch data from vaultier.org API or cache if nothing changed

        :param count: int
        :return: list
        """

        data = self._load_from_cache()
        if data:
            return data

        response = self._invoke_api(count)
        if response and response.status_code == 200:
            data = response.json()
            etag = response.headers['ETag']
            self._save(data, etag)
        return data


puller = NewsPuller(settings.VAULTIER.get('news_url'),
                    settings.VAULTIER.get('news_connection_timeout'),
                    settings.VAULTIER.get('news_cache_timeout'))



from rest_framework import serializers


class NewsSerializer(serializers.Serializer):
    id = serializers.IntegerField()
    text = serializers.CharField()
    title = serializers.CharField()
    link = serializers.URLField()
    published_at = serializers.DateTimeField()



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    depends_on = (('workspaces', '0001_initial'),
                  ('accounts', '0001_initial'))

    def forwards(self, orm):
        # Adding model 'Vault'
        db.create_table(u'vaultier_vault', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('deleted_at', self.gf('django.db.models.fields.DateTimeField')(null=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('color', self.gf('django.db.models.fields.CharField')(default='blue', max_length=30)),
            ('slug', self.gf('django.db.models.fields.CharField')(default='', max_length=255)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=1024, null=True, blank=True)),
            ('workspace', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['workspaces.Workspace'])),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['accounts.User'], on_delete=models.PROTECT)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'vaults', ['Vault'])


    def backwards(self, orm):
        # Deleting model 'Vault'
        db.delete_table(u'vaultier_vault')


    models = {
        u'accounts.user': {
            'Meta': {'object_name': 'User', 'db_table': "u'vaultier_user'"},
            'email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'unique': 'True', 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'nickname': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'public_key': ('django.db.models.fields.CharField', [], {'max_length': '1024'})
        },
        u'vaults.vault': {
            'Meta': {'object_name': 'Vault', 'db_table': "u'vaultier_vault'"},
            'color': ('django.db.models.fields.CharField', [], {'default': "'blue'", 'max_length': '30'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'on_delete': 'models.PROTECT'}),
            'deleted_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['workspaces.Workspace']"})
        },
        u'workspaces.workspace': {
            'Meta': {'object_name': 'Workspace', 'db_table': "u'vaultier_workspace'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'on_delete': 'models.PROTECT'}),
            'deleted_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['vaults']


from django.db import models
from django.db.models.deletion import CASCADE, PROTECT
from acls.business.managers import RoleManager
from libs.changes.changes import ChangesMixin, post_change
from vaultier.business.fields import ObjectReferenceTypeField
from vaultier.business.reference import ObjectReference
from .business.fields import RoleLevelField, AclDirectionField


class Acl(ObjectReference, models.Model):

    type = ObjectReferenceTypeField()
    to_workspace = models.ForeignKey('workspaces.Workspace', on_delete=CASCADE,
                                     null=True, blank=True)
    to_vault = models.ForeignKey('vaults.Vault', on_delete=CASCADE,
                                 null=True, blank=True)
    to_card = models.ForeignKey('cards.Card', on_delete=CASCADE, null=True,
                                blank=True)
    level = RoleLevelField()

    direction = AclDirectionField()
    role = models.ForeignKey('acls.Role', on_delete=CASCADE)
    user = models.ForeignKey('accounts.User', on_delete=CASCADE)

    class Meta:
        db_table = u'vaultier_acl'

    def get_target_string(self):
        if self.type == 100:
            return 'workspace:'+str(self.to_workspace.id)
        if self.type == 200:
            return 'vault:'+str(self.to_vault.id)
        if self.type == 300:
            return 'card:'+str(self.to_card.id)

    def __unicode__(self):
        return 'Acl({}): to:{} level:{} user: {}'.format(
            self.id,
            self.get_target_string(),
            self.level,
            self.user
        )


class Role(ChangesMixin, ObjectReference, models.Model):
    objects = RoleManager()

    type = ObjectReferenceTypeField()

    to_workspace = models.ForeignKey('workspaces.Workspace', on_delete=CASCADE,
                                     null=True, blank=True)
    to_vault = models.ForeignKey('vaults.Vault', on_delete=CASCADE,
                                 null=True, blank=True)
    to_card = models.ForeignKey('cards.Card', on_delete=CASCADE,
                                null=True, blank=True)
    member = models.ForeignKey('accounts.Member', on_delete=CASCADE)

    level = RoleLevelField()
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey('accounts.User', on_delete=PROTECT,
                                   related_name='roles_created')

    class Meta:
        db_table = u'vaultier_role'

    def save(self, *args, **kwargs):
        self.compute_type()
        return super(Role, self).save(*args, **kwargs)


def register_signals():
    post_change.connect(Role.objects.on_model, sender=Role)



from __future__ import absolute_import

from vaultier.statistics.statistics import StatisticsManager
from celery import shared_task


@shared_task()
def task_statistics_collector():
    """
    Collects statistics from database and sends them to vaultier.org
    """
    StatisticsManager.send_statistics()



from .fields import ObjectReferenceTypeField


class ObjectReference(object):

    def compute_type(self, force=None):
        if not self.type or force:
            type_set = False

            if self.to_workspace:
                if type_set:
                    raise RuntimeError('Role can reference only one object')
                self.type = ObjectReferenceTypeField.TYPE_WORKSPACE
                type_set = True

            if self.to_vault:
                if type_set:
                    raise RuntimeError('Role can reference only one object')
                self.type = ObjectReferenceTypeField.TYPE_VAULT
                type_set = True

            if self.to_card:
                if type_set:
                    raise RuntimeError('Role can reference only one object')
                self.type = ObjectReferenceTypeField.TYPE_CARD
                type_set = True

        if not self.type:
            raise RuntimeError('Role has no associated object')

    def get_object(self):
        self.compute_type()
        if self.type == ObjectReferenceTypeField.TYPE_WORKSPACE:
            return self.to_workspace
        if self.type == ObjectReferenceTypeField.TYPE_VAULT:
            return self.to_vault
        if self.type == ObjectReferenceTypeField.TYPE_CARD:
            return self.to_card

        raise RuntimeError('Role has no associated object')

    def set_object(self, object):
        self.to_workspace = None
        self.to_vault = None
        self.to_card = None
        self.type = None

        if not object:
            return

        if object.__class__.__name__ == 'Workspace':
            self.to_workspace = object

        if object.__class__.__name__ == 'Vault':
            self.to_vault = object

        if object.__class__.__name__ == 'Card':
            self.to_card = object

        self.compute_type()
        return



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from accounts.tests.api import register_api_call, auth_api_call
from libs.version.context import version_context_manager
from libs.version.manipulator import ACTION_CREATED, ACTION_UPDATED,\
    ACTION_SOFTDELETED
from vaultier.test.tools import AssertionsMixin
from versions.models import Version
from workspaces.tests.api import create_workspace_api_call, \
    update_workspace_api_call, delete_workspace_api_call


class WorkspaceVersionTest(AssertionsMixin,  TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(True)

    def create_workspace(self):
        # create user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token,  name='workspace').data

        return user1, user1token, workspace

    def test_workspace_010_create(self):
        user1, user1token,  workspace = list(self.create_workspace())

        #check version
        versions = Version.objects.filter(
            versioned_type__model='workspace',
            versioned_id=workspace.get('id')
        )

        # one version should be there
        self.assertEquals(versions.count(), 1)

        # compare version
        self.assert_model(versions[0], {
            'versioned_id': workspace.get('id'),
            'action_id': ACTION_CREATED,
        })

        # assert user has been stored with version
        self.assertEqual(versions[0].created_by.id, user1.get('id'))

        # compare revision data
        self.assert_dict(versions[0].revert_data, {})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'workspace')
        self.assertEqual(versions[0].versioned_related_id, workspace.get('id'))

    @unittest.skip("should be fixed asap")
    def test_workspace_020_update(self):
        user1, user1token,  workspace = list(self.create_workspace())

        workspace = update_workspace_api_call(
            user1token, workspace.get('id'), name='renamed_workspace',
            description="added_workspace_description"
        ).data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='workspace',
            versioned_id=workspace.get('id'),
            action_id=ACTION_UPDATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {
            'name': 'workspace',
            'description': None
        })

        workspace = update_workspace_api_call(
            user1token, workspace.get('id'),
            name='renamed_workspace_again',
            description="changed_workspace_description"
        ).data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='workspace',
            versioned_id=workspace.get('id'),
            action_id=ACTION_UPDATED
        )

        # two versions should be there
        self.assertEquals(versions.count(), 2)

        # compare revision data
        self.assert_dict(versions[1].revert_data, {
            'name': 'renamed_workspace',
            'description': 'added_workspace_description'
        })

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'workspace')
        self.assertEqual(versions[0].versioned_related_id, workspace.get('id'))

    def test_workspace_030_delete(self):
        user1, user1token,  workspace = list(self.create_workspace())

        #check version
        versions = Version.objects.filter(
            versioned_type__model='workspace',
            versioned_id=workspace.get('id'),
            action_id=ACTION_CREATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        delete_workspace_api_call(user1token, workspace.get('id'))

        #check version
        versions = Version.objects.filter(
            versioned_type__model='workspace',
            versioned_id=workspace.get('id'),
            action_id=ACTION_SOFTDELETED
        )

        # two versions should be there, created and deleted
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {'deleted_at': None})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'workspace')
        self.assertEqual(versions[0].versioned_related_id, workspace.get('id'))


def workspace_version_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        WorkspaceVersionTest))
    return suite



from django.db import models


class AbstractTreeIterator(object):
    def __init__(self, object, *args, **kwargs):
        self.object = object
        super(AbstractTreeIterator, self).__init__(*args, **kwargs)

    def get_object(self):
        return self.object

    def get_child_objects(self):
        raise NotImplementedError

    def get_parent_object(self):
        raise NotImplementedError

    def get_root_object(self):
        root = self
        o = root
        while o.get_parent_object():
            o = o.get_parent_object()
            root = o

        return root


class TreeIterableModelMixin(models.Model):
    class Meta:
        abstract = True

    tree_iterator_class = AbstractTreeIterator

    _tree_iterator = None

    def get_tree_iterator(self):
        if not self._tree_iterator:
            self._tree_iterator = self.tree_iterator_class(self)
        return self._tree_iterator



from rest_framework.permissions import BasePermission
from acls.business.fields import AclLevelField
from acls.business.permissions import has_object_acl


class CanManageVaultPermission(BasePermission):

    def has_object_permission(self, request, view, obj):

        if view.action == 'retrieve' or view.action == 'list':
            return has_object_acl(request.user, obj, AclLevelField.LEVEL_READ)

        if view.action == 'destroy':
            return has_object_acl(request.user, obj, AclLevelField.LEVEL_WRITE)

        if view.action == 'create':
            # check permission to workspace
            return has_object_acl(
                request.user, obj.workspace, AclLevelField.LEVEL_WRITE) or \
                has_object_acl(request.user, obj.workspace,
                               AclLevelField.LEVEL_CREATE)

        if view.action == 'update' or view.action == 'partial_update':
            result = True
            # check permission to vault
            result = result and has_object_acl(request.user, obj,
                                               AclLevelField.LEVEL_WRITE)
            # check permission to workspace
            result = result and has_object_acl(request.user, obj.workspace,
                                               AclLevelField.LEVEL_WRITE)
            return result

        return False



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_201_CREATED, HTTP_204_NO_CONTENT, \
    HTTP_200_OK
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call
from acls.models import Role
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from workspaces.models import Workspace
from workspaces.tests.api import create_workspace_api_call, \
    delete_workspace_api_call, list_workspaces_api_call, \
    retrieve_workspace_api_call, update_workspace_api_call


class ApiWorkspaceTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def create_workspace_and_user(self):

        # create user
        email = 'jan.misek@rclick.cz'
        user = register_api_call(email=email, nickname='Misan').data
        token = auth_api_call(email=email).data.get('token')

        # create workspace
        response = create_workspace_api_call(token, name='Workspace')

        # assert and assign
        self.assertEqual(
            response.status_code,
            HTTP_201_CREATED,
            format_response(response)
        )
        workspace = response.data

        return user, token, workspace

    def test_010_create_workspace(self):
        user, token, workspace = list(self.create_workspace_and_user())

        #assert single membership exists
        self.assertEqual(Member.objects.count(), 1)

        #assets role has been created together with workspace
        self.assertEqual(Role.objects.count(), 1)

    def test_020_delete_workspace(self):
        user, token, workspace = list(self.create_workspace_and_user())

        #assert workspace deletion
        response = delete_workspace_api_call(token, workspace.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

        #assert no workspace exists
        self.assertEqual(Workspace.objects.count(), 0)

    def test_030_list_and_retrieve_workspace(self):
        user, token, workspace = list(self.create_workspace_and_user())

        #assert listing
        response = list_workspaces_api_call(token)
        self.assertEqual(
            len(response.data),
            1
        )

        #assert listing
        response = retrieve_workspace_api_call(token, workspace.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

    def test_040_update_workspace(self):
        user, token, workspace = list(self.create_workspace_and_user())

        update_workspace_api_call(token, workspace.get('id'), name='updated')

        #assert updated
        response = retrieve_workspace_api_call(token, workspace.get('id'))
        self.assertEqual(
            response.data.get('name'),
            'updated',
        )


def workspace_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        ApiWorkspaceTest))
    return suite



from django.contrib.auth.models import AbstractBaseUser
from django.db import models
from django.db.models.deletion import CASCADE, PROTECT
from django.db.models.signals import pre_save, post_save
from accounts.business.fields import MemberStatusField
from libs.changes.changes import ChangesMixin
from libs.lowercasefield.lowercasefield import LowerCaseCharField
from .business.managers import UserManager, LostKeyManager, MemberManager, \
    TokenManager
from django.utils import timezone
import random


class User(ChangesMixin, AbstractBaseUser):

    nickname = models.CharField(max_length=255, blank=False, null=False)
    public_key = models.CharField(max_length=1024)
    email = LowerCaseCharField(max_length=255, unique=True)
    is_active = models.BooleanField(default=True)

    objects = UserManager()

    REQUIRED_FIELDS = []
    USERNAME_FIELD = 'email'

    class Meta:
        db_table = u'vaultier_user'


class Token(ChangesMixin, models.Model):

    TOKEN_LENGTH = 64
    """
    Length of generated token
    """
    token = models.CharField(max_length=TOKEN_LENGTH, unique=True,
                             db_index=True)
    user = models.ForeignKey('accounts.User', on_delete=CASCADE)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    last_used_at = models.DateTimeField(null=True)

    objects = TokenManager()

    class Meta:
        db_table = u'vaultier_token'

    def save(self, *args, **kwargs):
        if not self.token:
            self.token = self.generate_token()
        if not self.last_used_at:
            self.last_used_at = timezone.now()
        return super(Token, self).save(*args, **kwargs)

    def generate_token(self):
        chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVW' \
                'XYZ!"#$%&\()*+,-./:;<=>?@[\]^_`{|}~'
        unique = ''.join(random.choice(chars) for i in range(
            Token.TOKEN_LENGTH))
        return unique

    def __unicode__(self):
        return self.key


class LostKey(models.Model):
    objects = LostKeyManager()

    hash = models.TextField(null=False)
    created_by = models.ForeignKey('accounts.User', on_delete=PROTECT,
                                   related_name='distracted')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    expires_at = models.DateTimeField(null=False)
    used = models.BooleanField(default=False)

    class Meta:
        db_table = u'vaultier_lost_key'


class Member(ChangesMixin, models.Model):

    workspace = models.ForeignKey(
        'workspaces.Workspace', related_name='membership', on_delete=CASCADE)

    user = models.ForeignKey(
        'accounts.User', on_delete=CASCADE, null=True,
        related_name='membership')
    invitation_hash = models.CharField(max_length=64, null=True, unique=True)
    invitation_email = LowerCaseCharField(max_length=1024, null=True)
    workspace_key = models.CharField(max_length=4096)
    status = MemberStatusField()
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    created_by = models.ForeignKey(
        'accounts.User', on_delete=PROTECT, related_name='members_created')
    objects = MemberManager()

    class Meta:
        db_table = u'vaultier_member'

    def is_invitation(self):
        if self.status == MemberStatusField.STATUS_INVITED:
            return True
        return False

    def save(self, *args, **kwargs):
        if not self.invitation_hash:
            self.invitation_hash = Member.objects.generate_invitation_hash()
        return super(Member, self).save(*args, **kwargs)


def register_signals():
    pre_save.connect(LostKey.objects.on_pre_save, sender=LostKey)
    post_save.connect(LostKey.objects.send_notification, sender=LostKey)
    post_save.connect(
        Member.objects.send_transfer_workspace_key_info, sender=Member)



from django.core.urlresolvers import reverse

# todo: RecoverTypeField?
# from vaultier.models.lostkey.fields import RecoverTypeField
from vaultier.test.tools import VaultierAPIClient


def create_lost_keys_api_call(email, **kwargs):
    """
    Call to lost_key create view
    :param email: user email
    :param kwargs:
    :return: Response
    """
    client = VaultierAPIClient()
    kwargs['email'] = email
    url = reverse('lost_keys-list')
    response = client.post(url, data={'email': email}, kwargs=kwargs)
    return response


def update_lost_key_api_rebuild_call(lost_key_id, auth_hash=None,
                                     public_key=None):
    """
    Call to update view with parameter recover_type set to
    RecoverTypeField.REBUILD
    :param lost_key_id: int
    :param auth_hash: str
    :param public_key: str
    :return: Response
    """
    client = VaultierAPIClient()
    url = reverse('lost_keys-detail',
                  args=(lost_key_id,)) + '?hash=' + auth_hash
    # todo: RecoverTypeField?
    # return client.put(url, data={'public_key': public_key,
    #                              'recover_type': RecoverTypeField.REBUILD})
    return client.put(url, data={'public_key': public_key,
                                 'recover_type': None})


def update_lost_key_api_disable_call(lost_key_id, auth_hash=None,
                                     public_key=None):
    """
    Call to update view with parameter recover_type set to
    RecoverTypeField.DISABLE
    :param lost_key_id: int
    :param auth_hash: str
    :param public_key: str
    :return: Response
    """
    client = VaultierAPIClient()
    url = reverse('lost_keys-detail',
                  args=(lost_key_id,)) + '?hash=' + auth_hash
    # todo: RecoverTypeField?
    # return client.put(url, data={'public_key': public_key,
    #                              'recover_type': RecoverTypeField.DISABLE})
    return client.put(url, data={'public_key': public_key,
                                 'recover_type': None})


def retrieve_lost_key_api_call(lost_key_id, auth_hash=None):
    """
    Call to retrieve view
    :param lost_key_id: int
    :param auth_hash: str
    :return: Response
    """
    client = VaultierAPIClient()
    url = reverse('lost_keys-detail',
                  args=(lost_key_id,)) + '?hash=' + auth_hash
    return client.get(url)



from libs.version.condition import RequiredFieldEventCondition
from libs.version.manipulator import register_manipulator_signal
from libs.changes.changes import SOFT_DELETE, INSERT, UPDATE
from libs.version.manipulator import ModelCreatedManipulator, \
    ModelUpdatedManipulator, ModelSoftDeletedManipulator, \
    register_manipulator_class
from django.db.models.loading import get_model
from vaults.models import Vault


def register_signals():
    register_manipulator_class('vault_created_manipulator',
                               ModelCreatedManipulator)
    register_manipulator_class('vault_updated_manipulator',
                               ModelUpdatedManipulator)
    register_manipulator_class('vault_deleted_manipulator',
                               ModelSoftDeletedManipulator)

    version_cls = get_model('versions', 'Version')

    register_manipulator_signal(
        version_cls=version_cls,
        manipulator_id='vault_deleted_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Vault,
            required_fields=['deleted_at'],
            required_event=SOFT_DELETE,
        )
    )

    register_manipulator_signal(
        version_cls=version_cls,
        manipulator_id='vault_updated_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Vault,
            required_fields=['name', 'description'],
            required_event=UPDATE,
        )
    )

    register_manipulator_signal(
        version_cls=version_cls,
        manipulator_id='vault_created_manipulator',
        condition=RequiredFieldEventCondition(
            required_sender=Vault,
            required_fields=None,
            required_event=INSERT,
        )
    )



from accounts.business.fields import MemberStatusField
from acls.business.fields import RoleLevelField, AclDirectionField
from acls.models import Acl, Role
from acls.business.perms.strategy import ReadAclStrategy, WriteAclStrategy, \
    CreateAclStrategy


class CreateRoleMaterializer(object):
    role = None

    def __init__(self, role):
        self.role = role

    def get_strategy(self):
        strategy = None
        if self.role.level == RoleLevelField.LEVEL_CREATE:
            strategy = CreateAclStrategy
        if self.role.level == RoleLevelField.LEVEL_READ:
            strategy = ReadAclStrategy
        if self.role.level == RoleLevelField.LEVEL_WRITE:
            strategy = WriteAclStrategy
        if not strategy:
            raise RuntimeError('Cannot find ACL strategy for role level {}'.
                               format(self.role.level))
        return strategy()

    def acl_for_object(self, object, direction):

        if not self.role.member.user:
            raise RuntimeError(''
                               'Acl could be materialized upon conrete '
                               'member. Not the invited one without user set')

        strategy = self.get_strategy()

        if direction == AclDirectionField.DIR_UP:
            return strategy.acl_for_parent(self.role, object)

        if direction == AclDirectionField.DIR_DOWN:
            return strategy.acl_for_child(self.role, object)

        if direction == AclDirectionField.DIR_CURRENT:
            return strategy.acl_for_object(self.role, object)

        raise RuntimeError('Unknown ACL direction {}'.format(direction))

    def materialize_parents(self, object):
        acls = []
        parent = object.get_tree_iterator().get_parent_object()
        if parent:
            acls.extend(self.acl_for_object(parent, AclDirectionField.DIR_UP))
            acls.extend(self.materialize_parents(parent))
        return acls

    def materialize_childs(self, object):
        acls = []
        childs = object.get_tree_iterator().get_child_objects()
        for child in childs:
            acls.extend(self.acl_for_object(child, AclDirectionField.DIR_DOWN))
            acls.extend(self.materialize_childs(child))
        return acls

    def materialize_current(self, object):
        acls = []
        acls.extend(self.acl_for_object(object, AclDirectionField.DIR_CURRENT))
        return acls

    def materialize(self, object):
        acls = []

        # materialize current
        acls.extend(self.materialize_current(object))

        # materialize parent objects
        acls.extend(self.materialize_parents(object))

        # materialize child objects
        acls.extend(self.materialize_childs(object))

        # save materialized
        saver = MaterializationSaver()
        saver.save_materialized(acls)


class MaterializationSaver(object):

    def save_materialized(self, acls):
        saved = []

        # find existing acls
        # this should be optimized, not do million db queries
        for acl in acls:
            try:
                Acl.objects.get(
                    role=acl.role,
                    direction=acl.direction,
                    level=acl.level,
                    to_workspace=acl.to_workspace,
                    to_vault=acl.to_vault,
                    to_card=acl.to_card
                )

            except Acl.DoesNotExist:
                # standard behaviour acls not found, so create new
                acl.save()
                saved.append(acl)

            except Acl.MultipleObjectsReturned:
                # more acls returned, this should not happen, so delete them
                # and create new
                Acl.objects.filter(
                    role=acl.role,
                    to_workspace=acl.to_workspace,
                    to_vault=acl.to_vault,
                    to_card=acl.to_card
                ).delete()

                acl.save()
                saved.append(acl)


class UpdateRoleLevelMaterializer(object):
    def __init__(self, role):
        self.role = role

    def materialize(self):
        Acl.objects.filter(
            role=self.role
        ).delete()

        if self.role.member.user:
            materializer = CreateRoleMaterializer(self.role)
            materializer.materialize(self.role.get_object())


class UpdateRoleMemberMaterializer(object):
    def __init__(self, role):
        self.role = role

    def materialize(self):
        Acl.objects.filter(
            role=self.role
        ).delete()

        materializer = CreateRoleMaterializer(self.role)
        materializer.materialize(self.role.get_object())


class UpdateMemberUserMaterializer(object):
    def __init__(self, member):
        self.member = member

    def materialize(self):
        member = self.member
        for role in member.role_set.all():
            # first delete old acls
            Acl.objects.filter(
                role=role,
            ).delete()

            # materialize
            materializer = CreateRoleMaterializer(role)
            materializer.materialize(role.get_object())


class InsertedObjectMaterializer(object):
    """
    Materializes acls for just inserted object
    """
    def __init__(self, object):
        self.object = object

    def materialize_role(self):
        """
        adds WRITE role for objects created by user based of parent
        CREATE roles
        """
        parent = self.object.get_tree_iterator().get_parent_object()
        while parent:
            for parent_role in parent.role_set.all():
                if parent_role.member.user and \
                   parent_role.member.user.id == self.object.created_by.id and\
                   parent_role.level == RoleLevelField.LEVEL_CREATE:
                        role = Role()
                        role.member = parent_role.member
                        role.created_by = role.member.user
                        role.level = RoleLevelField.LEVEL_WRITE
                        role.set_object(self.object)
                        role.save()
                        return

            parent = parent.get_tree_iterator().get_parent_object()

    # materialize roles
    def materialize(self):
        parent = self.object.get_tree_iterator().get_parent_object()
        roles = []
        acls = []

        # collect all parent roles
        while parent:
            for role in parent.role_set.all():
                roles.append(role)
            parent = parent.get_tree_iterator().get_parent_object()

        # rematerialize all childs roles including
        # rematerialization on current child
        for role in roles:
            if not role.member.status == MemberStatusField.STATUS_INVITED:
                rm = CreateRoleMaterializer(role)
                acls.extend(rm.materialize_childs(role.get_object()))

        # save materialized
        saver = MaterializationSaver()
        saver.save_materialized(acls)

        # rematerialize roles in role_set of object
        for role in self.object.role_set.all():
            rm = CreateRoleMaterializer(role)
            rm.materialize(self.object)


class MovedObjectMaterializer(object):
    def __init__(self, object):
        self.object = object

    def materialize(self):
        # delete all acls related to object
        self.object.acl_set.all().delete()

        # delete all acls related to object roles,
        # including acls of previous parents
        Acl.objects.filter(role__in=self.object.role_set.all()).delete()

        # object previous acls are now cleared
        # rematerialize object as newly created
        m = InsertedObjectMaterializer(self.object)
        m.materialize()



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    depends_on = (('workspaces', '0001_initial'),)

    def forwards(self, orm):
        # Adding field 'Member.workspace'
        db.add_column(u'vaultier_member', 'workspace',
                      self.gf('django.db.models.fields.related.ForeignKey')(default=1, related_name='membership', to=orm['workspaces.Workspace']),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Member.workspace'
        db.delete_column(u'vaultier_member', 'workspace_id')


    models = {
        u'accounts.lostkey': {
            'Meta': {'object_name': 'LostKey', 'db_table': "u'vaultier_lost_key'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'distracted'", 'on_delete': 'models.PROTECT', 'to': u"orm['accounts.User']"}),
            'expires_at': ('django.db.models.fields.DateTimeField', [], {}),
            'hash': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'used': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        u'accounts.member': {
            'Meta': {'object_name': 'Member', 'db_table': "u'vaultier_member'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'members_created'", 'on_delete': 'models.PROTECT', 'to': u"orm['accounts.User']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'invitation_email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'max_length': '1024', 'null': 'True'}),
            'invitation_hash': ('django.db.models.fields.CharField', [], {'max_length': '64', 'unique': 'True', 'null': 'True'}),
            'status': ('accounts.business.fields.MemberStatusField', [], {'default': '3'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'membership'", 'null': 'True', 'to': u"orm['accounts.User']"}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'membership'", 'to': u"orm['workspaces.Workspace']"}),
            'workspace_key': ('django.db.models.fields.CharField', [], {'max_length': '4096'})
        },
        u'accounts.token': {
            'Meta': {'object_name': 'Token', 'db_table': "u'vaultier_token'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_used_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'token': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64', 'db_index': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']"})
        },
        u'accounts.user': {
            'Meta': {'object_name': 'User', 'db_table': "u'vaultier_user'"},
            'email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'unique': 'True', 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'nickname': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'public_key': ('django.db.models.fields.CharField', [], {'max_length': '1024'})
        },
        u'workspaces.workspace': {
            'Meta': {'object_name': 'Workspace', 'db_table': "u'vaultier_workspace'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'on_delete': 'models.PROTECT'}),
            'deleted_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['accounts']


import hmac
import uuid
from datetime import datetime, timedelta
from django.contrib.auth.models import BaseUserManager
from django.db.models import Q
from django.db.models.loading import get_model
from django.db.models.manager import Manager
from django.utils import timezone
from hashlib import sha1
from django.utils.timezone import utc
from django.conf import settings
from accounts.business.fields import MemberStatusField
from accounts.business.mailer import LostKeyMailer
from django.db.models.aggregates import Count
from workspaces.business.mailer import InvitationMailer, \
    WorkspaceKeyTransferMailer


class UserManager(BaseUserManager):

    def create_user(self, username, email=None, password=None, **extra_fields):
        now = timezone.now()
        if not email:
            raise ValueError('The given email must be set')
        email = UserManager.normalize_email(email)
        user = self.model(username=username, email=email,
                          is_staff=False, is_active=True, is_superuser=False,
                          last_login=now, **extra_fields)

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, username, email, password, **extra_fields):
        u = self.create_user(username, email, password, **extra_fields)
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True
        u.save(using=self._db)
        return u


class LostKeyManager(Manager):

    def on_pre_save(self, sender, instance=None, *args, **kwargs):
        if not instance.id:
            self._generate_hash(instance=instance)
            self._generate_expiration_time(instance=instance)

    def _generate_hash(self, instance=None):
        unique_hash = uuid.uuid4()
        instance.hash = hmac.new(unique_hash.bytes, digestmod=sha1).hexdigest()

    def _generate_expiration_time(self, instance=None):
        expiration_time = settings.VAULTIER.get(
            'lostkey_hash_expiration_time')
        instance.expires_at = timezone.now().replace(tzinfo=utc) + \
            timedelta(milliseconds=expiration_time)

    def send_notification(self, sender, instance=None, *args, **kwargs):
        """
        Sends email to user with the valid url for update the public key
        :param sender:
        :param instance:
        :param args:
        :param kwargs:
        :return: None
        """
        if not instance.used:
            sender = LostKeyMailer(object=instance)
            sender.send()

    def disable_lost_key(self, user):
        """
        Soft delete nan shared workspaces of a given user
        :param user:
        :return:
        """
        self._disable_memberships(user)
        self._softdelete_unrecoverable_workspaces(user)

    @classmethod
    def _disable_memberships(cls, user):
        """
        Set all membership of given user to
        MemberStatusField.STATUS_MEMBER_BROKEN
        :param user:
        :return:
        """
        get_model('accounts', 'Member').objects.filter(user=user).update(
            status=MemberStatusField.STATUS_MEMBER_BROKEN)

    @classmethod
    def find_workspace_is_recoverable(cls, workspace_id, user):
        """
        Return True if the workspace is recoverable.
        A work space is recoverable when it share among any user and those
        users have the status set to MemberStatusField.STATUS_MEMBER
        :param workspace_id: int
        :param user: vaultier.models.user.model.User
        :return: bool
        """
        return 0 < get_model('accounts', 'Member').objects.filter(
            workspace_id=workspace_id,
            status=MemberStatusField.STATUS_MEMBER).exclude(user=user).count()

    def rebuild_lost_key(self, user):
        """
        :param user:
        :return:
        """
        self._set_unrecoverable_workspaces_broken(user)
        self._set_status_member_without_workspace_key(user)
        self._softdelete_unrecoverable_workspaces(user)

    def _set_status_member_without_workspace_key(self, user):
        """
        Update status of shared memberships for a given user
        :param user:
        :return:
        """
        unrecoverable_workspaces = self._get_unrecoverable_workspaces(user)
        get_model('accounts', 'Member').objects.filter(user=user) \
            .exclude(
                workspace_id__in=unrecoverable_workspaces.
                values_list('pk', flat=True)
            ).update(
                status=MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY,
                workspace_key=''
            )

    def _set_unrecoverable_workspaces_broken(self, user):
        """
        Set the membership status to broken
        just for nonrecoverable workspaces
        :param user:
        :return:
        """
        unrecoverable_workspaces = self._get_unrecoverable_workspaces(user)
        get_model('accounts', 'Member').objects.filter(
            user=user,
            workspace_id__in=unrecoverable_workspaces
            .values_list('pk', flat=True)) \
            .update(status=MemberStatusField.STATUS_MEMBER_BROKEN)

    def _softdelete_unrecoverable_workspaces(self, user):
        """
        Deletes workspaces whe
        :param user:
        :return:
        """
        unrecoverable_workspaces = self._get_unrecoverable_workspaces(user)
        get_model('workspaces', 'Workspace').bulk_delete(
            unrecoverable_workspaces)

    def _get_unrecoverable_workspaces(self, user):
        """
        Filter Workspace where the user is the only member
        :param user:
        :return: QuerySet
        """
        member_cls = get_model('accounts', 'Member')
        return get_model('workspaces', 'Workspace').objects.filter(
            pk__in=member_cls.objects.filter(user=user).values_list(
                'workspace_id', flat=True)
        ).annotate(is_recoverable=Count('membership')).exclude(
            is_recoverable__gt=1)


class MemberManager(Manager):

    def all_for_user(self, user):
        from workspaces.models import Workspace

        workspaces = Workspace.objects.all_for_user(user)
        return get_model('accounts', 'Member').objects.\
            filter(workspace__in=workspaces)

    def all_to_transfer_keys(self, user):

        workspaces_where_user_is_member_with_keys = \
            get_model('workspaces', 'Workspace').objects.filter(
                membership__status=MemberStatusField.STATUS_MEMBER,
                membership__user=user
            ).distinct()

        # has key can see all without workspace key
        # OR
        # has not key, can work only with own workspace key

        query = self.all_for_user(user).filter(
            Q(
                Q(workspace__in=workspaces_where_user_is_member_with_keys)
                &
                Q(status=MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY)
            )
            |
            Q(
                user=user
            )
        ).distinct()

        return query

    def generate_invitation_hash(self):
        def get_unique():
            unique = uuid.uuid4()
            return hmac.new(unique.bytes, digestmod=sha1).hexdigest()

        hash = get_unique()
        while self.filter(invitation_hash=hash):
            hash = get_unique()

        return hash

    def get_concrete_member_to_workspace(self, workspace, user):
        member = None
        try:
            member = self.filter(
                workspace=workspace,
                user=user,
                status__gt=MemberStatusField.STATUS_INVITED
            )[0]
        except:
            pass

        return member

    def join_member(self, source, target):
        for role in source.role_set.all():
            role.member = target
            role.save()
        source.delete()
        self.remove_role_duplicatates(target)

    def remove_role_duplicatates(self, member):
        # @todo: optimize this, many queries done
        for role in member.role_set.all():
            delete = member.role_set.filter(
                to_workspace=role.to_workspace,
                to_vault=role.to_vault,
                to_card=role.to_card,
                level__gte=role.level
            ).exclude(id=role.id).count() >= 1
            if delete:
                role.delete()

    def accept_invitation(self, member, user):
        workspace = member.workspace
        existing_member = self.get_concrete_member_to_workspace(
            workspace, user
        )

        # user membership does not exists, or it is only invite
        if not existing_member:
            member.user = user
            member.status = MemberStatusField.\
                STATUS_MEMBER_WITHOUT_WORKSPACE_KEY
            member.save()

        # membership exists, we need to join current membership to existing
        else:
            self.join_member(member, existing_member)
            member = existing_member

        # reload member
        member = get_model('accounts', 'Member').objects.get(id=member.id)

        return member

    def invite(self, member, send=True, resend=True):

        member_cls = get_model('accounts', 'Member')

        try:
            # find invitation
            member = member_cls.objects.get(
                status=MemberStatusField.STATUS_INVITED,
                invitation_email=member.invitation_email,
                workspace=member.workspace
            )
            if resend:
                self.send_invitation(member)

        # invitation not found so create new
        except member_cls.DoesNotExist:
            member.id = None
            member.status = MemberStatusField.STATUS_INVITED
            member.save()
            if send:
                self.send_invitation(member)

        # todo: Member.MultipleObjectsFound

        return member

    def send_invitation(self, member):
        """
        Sends an invitation email
        :param member: Member
        :return: None
        """
        invitation_sender = InvitationMailer(object=member)
        invitation_sender.send()

    def send_transfer_workspace_key_info(self, sender, instance=None, *args,
                                         **kwargs):
        """

        :param sender:
        :param instance:
        :param args:
        :param kwargs:
        :return:
        """
        if instance.status == MemberStatusField.STATUS_MEMBER and \
                instance.invitation_email and instance.invitation_hash:
            # the user has been invited and the workspace_key was set
            sender = WorkspaceKeyTransferMailer(object=instance)
            sender.send()

    def clean_old_invitations(self):
        lifetime_in_days = settings.VAULTIER.get("invitation_lifetime")
        expired_date = timezone.now() - timedelta(days=lifetime_in_days)
        self.filter(status=MemberStatusField.STATUS_INVITED,
                    created_at__lt=expired_date
                    ).delete()


class TokenManager(Manager):

    def clean_old_tokens(self):
        token_lifetime = settings.VAULTIER.get('authentication_token_lifetime')
        expired_date = datetime.now() - timedelta(hours=token_lifetime)
        self.filter(last_used_at__lte=expired_date).delete()



from .card import card_suite
from .card_perms import card_perms_suite
from .card_softdelete import card_softdelete_suite
from .card_version import card_version_suite
from django.utils.unittest.suite import TestSuite


def suite():
    s = TestSuite()
    s.addTest(card_suite())
    s.addTest(card_perms_suite())
    s.addTest(card_softdelete_suite())
    s.addTest(card_version_suite())
    return s



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_403_FORBIDDEN, HTTP_200_OK, \
    HTTP_204_NO_CONTENT
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call
from acls.business.fields import RoleLevelField
from acls.tests.api import create_role_api_call
from cards.tests.api import create_card_api_call, delete_card_api_call, \
    retrieve_card_api_call, update_card_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from vaults.tests.api import create_vault_api_call, retrieve_vault_api_call
from workspaces.tests.api import create_workspace_api_call, \
    accept_invitation_api_call, retrieve_workspace_api_call, \
    list_workspaces_api_call, delete_workspace_api_call


class ApiCardPermsTest(TransactionTestCase):

    def test_000_vault_anonymous_access(self):
        response = create_vault_api_call(None, name="card_in_workspace",
                                         workspace=None)
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_001_create_delete_card_as_anonymous(self):
        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='Workspace').data

        # create vault
        vault = create_vault_api_call(user1token, name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="card_in_vault",
                                    vault=vault.get('id')).data

        #delete card as anonymous
        response = delete_card_api_call(None, card.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    def test_010_create_card_in_vault_without_acl_to_vault(self):
        # create user1
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault_in_workspace',
                                       workspace=workspace1.get('id')).data

        # user2 tries to create vault in workspace of user 2 which he
        # has no access to
        response = create_card_api_call(user2token, name="card_in_vault",
                                        vault=vault1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_020_create_card_and_and_check_permissions(self):
        # create user1
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        user2 = register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='Workspace').data

        # create vault
        vault1 = create_vault_api_call(user1token, name="vault_in_workspace",
                                       workspace=workspace1.get('id')).data

        # create card
        card1 = create_card_api_call(user1token, name='card_in_vault',
                                     vault=vault1.get('id')).data

        # user1 invites user and grant to user role READ to card1
        user2member = invite_member_api_call(user1token,
                                             email=user2.get('email'),
                                             workspace=workspace1.get(
                                                 'id')).data
        user2hash = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2accepted = accept_invitation_api_call(user2token,
                                                   pk=user2member.get('id'),
                                                   hash=user2hash)
        user2role = create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_card=card1.get('id'),
            level=RoleLevelField.LEVEL_READ
        )

        #user2 tries to read card, should be allowed
        response = retrieve_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to read vault, should be allowed
        response = retrieve_vault_api_call(user2token, vault1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to read workspace, should be allowed
        response = retrieve_workspace_api_call(user2token,
                                               workspace1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to delete card, should be forbidden
        response = delete_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        #user2 tries to list workspace of vault1 of card1, should be allowed
        response = list_workspaces_api_call(user2token)
        self.assertEqual(
            len(response.data),
            1,
            format_response(response)
        )

        #user2 tries to delete workspace of vault1, should be forbidden
        response = delete_workspace_api_call(user2token,
                                             vault1.get('workspace'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        #user1 grants write to workspace, user2 should be able to delete card
        create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_workspace=workspace1.get('id'),
            level=RoleLevelField.LEVEL_WRITE
        )

        #user2 tries to delete card, should be allowed
        response = delete_card_api_call(user2token, card1.get('id'))
        self.assertEqual(response.status_code, HTTP_204_NO_CONTENT,
                         format_response(response))

    def test_030_retrieve_forbidden(self):
        # create user1
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault_in_workspace',
                                       workspace=workspace1.get('id')).data

        # create card1
        card1 = create_card_api_call(user1token, name="card_in_vault",
                                     vault=vault1.get('id')).data

        # user 2 tries to retrieve card1 should be forbidden
        response = retrieve_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_040_move(self):
        # create user1
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        user2 = register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(user1token,
                                               name='workspace').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault1_in_workspace',
                                       workspace=workspace1.get('id')).data

        # user1 creates vault2
        vault2 = create_vault_api_call(user1token, name='vault2_in_workspace',
                                       workspace=workspace1.get('id')).data

        # user1 creates card1 inside vault1
        card1 = create_card_api_call(user1token, name="card1_in_vault1",
                                     vault=vault1.get('id')).data

        # invite user 2 to vault2
        user2member = invite_member_api_call(user1token,
                                             email=user2.get('email'),
                                             workspace=workspace1.get(
                                                 'id')).data
        user2hash = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        accept_invitation_api_call(user2token,
                                   pk=user2member.get('id'),
                                   hash=user2hash)

        create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_vault=vault2.get('id'),
            level=RoleLevelField.LEVEL_READ
        )

        # user 2 tries to retrieve card1 should not be allowed
        response = retrieve_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # user 2 tries to retrieve vault2 should be allowed
        response = retrieve_vault_api_call(user2token, vault2.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user1 moves card to vault2
        update_card_api_call(user1token, card1.get('id'),
                             vault=vault2.get('id')).data

        # card1 should be now accessible to user 2
        response = retrieve_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # user2 is granted access directly to card
        create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_card=card1.get('id'),
            level=RoleLevelField.LEVEL_READ
        )

        # user1 moves card back to vault1
        update_card_api_call(user1token, card1.get('id'),
                             vault=vault1.get('id')).data

        # user2 should be able to access vault and card
        response = retrieve_card_api_call(user2token, vault1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )
        response = retrieve_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )


def card_perms_suite():
    suite = TestSuite()
    suite.addTest(
        unittest.TestLoader().loadTestsFromTestCase(ApiCardPermsTest))
    return suite



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def search_api_call(token, query=None):
    url = reverse('search-search')
    client = VaultierAPIClient()
    client.token(token)

    data = {
        'query': query
    }

    response = client.get(url, data)
    return response



from django.forms.models import model_to_dict
from rest_framework.test import APIClient
import json
from django.conf import settings


class FileAccessMixin(object):

    def open_file(self, filename):
        file = open('{}/test/fixtures/{}'.format(
            settings.PROJECT_PATH, filename))
        return file

    def read_file(self, filename):
        file = self.open_file(filename)
        data = file.read()
        file.close()
        return data


class AssertionsMixin(object):

    def assert_model(self, version, required):
        version = model_to_dict(version)
        return self.assert_dict(version, required)

    def assert_dict(self, compared, required):
        for key in required.keys():
            if not key in compared:
                raise self.failureException('Missing key "{}"'.format(key))
            if compared[key] != required[key]:
                raise self.failureException('key "{}" not same'.format(key))


class VaultierAPIClient(APIClient):

    def token(self, token):
        self.credentials(HTTP_X_VAULTIER_TOKEN=token)


def format_response(response):
    data = json.dumps(response.data, default=lambda obj: None)
    code = str(response.status_code)
    return 'code:' + code + ', json:' + data



import os.path
import sys
from celery.schedules import crontab

DEBUG = False

TEMPLATE_DEBUG = DEBUG

BASE_DIR = os.path.join(
    os.path.dirname(os.path.dirname(__file__)),
    '..'
)

PROJECT_PATH = os.path.split(os.path.abspath(os.path.dirname(__file__)))[0]


DATABASES = {
    'default': {
        # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'ENGINE': 'django.db.backends.mysql',
        # 'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'vaultier',  # Or path to database file if using sqlite3.
        # The following settings are not used with sqlite3:
        'USER': 'root',
        'PASSWORD': 'password',
        # Empty for localhost through domain sockets or '127.0.0.1'
        # for localhost through TCP.
        'HOST': '',
        'PORT': '',  # Set to empty string for default.
    }
}
# Covers regular testing and django-coverage
if 'test' in sys.argv or 'test_coverage' in sys.argv:
    DATABASES['default']['ENGINE'] = 'django.db.backends.sqlite3'


# Hosts/domain names that are valid for this site; required if DEBUG is False
# See https://docs.djangoproject.com/en/1.5/ref/settings/#allowed-hosts
ALLOWED_HOSTS = []

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'UTC'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = os.path.join(PROJECT_PATH, 'media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = os.path.join(PROJECT_PATH, "static")

# URL prefix for static files.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = '/static/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    ('vaultier', os.path.join(PROJECT_PATH, 'scripts', 'dist')),
)

SITE_URL = 'http://localhost:8000/'

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = ''

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
)

ROOT_URLCONF = 'vaultier.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = 'vaultier.wsgi.application'

# Template settings
TEMPLATE_DIRS = (
    os.path.join(BASE_DIR, 'vaultier/tempates'),
    os.path.join(BASE_DIR, 'vaultier/business'),

    # Put strings here, like "/home/html/django_templates" or
    # "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

REST_FRAMEWORK = {
    'DEFAULT_FILTER_BACKENDS': (
        'rest_framework.filters.DjangoFilterBackend',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
    ),
    'EXCEPTION_HANDLER': 'rest_framework.views.exception_handler',
    'TEST_REQUEST_RENDERER_CLASSES': (
        'rest_framework.renderers.MultiPartRenderer',
        'rest_framework.renderers.JSONRenderer',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'accounts.business.authentication.TokenAuthentication',
    ),
}

COMPRESS_ENABLED = 1

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'django.contrib.staticfiles',
    'raven.contrib.django.raven_compat',
    'kombu.transport.django',
    'vaultier',
    'south',
    'rest_framework',
    'accounts',
    'acls',
    'cards',
    'search',
    'secrets',
    'vaults',
    'versions',
    'workspaces',
    'slugs',
    'news'
)

#Django cache settings
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'vaultier_cache',
    }
}


# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'default': {
            'format': '%(asctime)-15s %(name)-7s %(levelname)s: %(message)s',
            'datefmt': '%m/%d/%Y %H:%M:%S',
        },
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

EMAIL_HOST = 'localhost'
EMAIL_PORT = 25
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = False

AUTH_USER_MODEL = 'accounts.User'

TEST_RUNNER = 'django.test.simple.DjangoTestSuiteRunner'

EMBER_TPL_DIR = os.path.join(PROJECT_PATH, 'static/js/module')
EMBER_TPL_CMPDIR = os.path.join(PROJECT_PATH, 'static/tpl')
EMBER_TPL_MASK = "\w+.hbs$"

VAULTIER = {
    # sentry key to be used for loggin errors on the frontend
    'raven_key': '',
    # 'email@example.com' to send all emails to this address
    'dev_mail_to': False,
    # True to use/generate same key for all users
    'dev_shared_key': False,
    # 1 hour in milliseconds used to calculate the expiration time
    # on api.lostkey module
    'lostkey_hash_expiration_time': 60*60*1000,
    # Default email address from which we send emails
    'from_email': 'noreply@vaultier.org',
    # Max difference between timestamp from server and from front-end
    # (in seconds)
    'login_safe_timestamp': 15,
    # token lifetime (in hours)
    'authentication_token_lifetime': 2,
    # last_used_at will be renewed after some interval (in minutes)
    'authentication_token_renewal_interval': 1,
    # invitation lifetime (in hours)
    'invitation_lifetime': 7,
    # When anonymous usage statistics is enabled, Vaultier periodically
    # anonymously notifies its maintainer and developer about running
    # instance of Vaultier.
    # As Vaultier is Open Source we kindly ask you to leave this option
    # enabled and let us know how many instances are deployed.
    # We send these statistics data: count of workspaces, vaults, cards,
    # secrets, users and members
    'allow_anonymous_usage_statistics': True,
    'registration_allow': True,
    # Vaultier blog news API endpoint. Must end with trailing slash
    'news_url': 'http://vaultier.org/api/entries/',
    # For how long wait for response in seconds.
    'news_connection_timeout': 2,

    # For how long news should be hold in cache in seconds. Vaultier API
    # provides ETag functionality, so you can increase this
    # value whatever you want.
    'news_cache_timeout': 60*10,
}

#celery broker
BROKER_URL = 'django://'
CELERY_ACCEPT_CONTENT = ['json', 'pickle']
CELERY_TASK_SERIALIZER = 'pickle'
CELERY_RESULT_SERIALIZER = 'pickle'
CELERYBEAT_SCHEDULE = {
    'garbage_collector_tokens': {
        'task': 'accounts.tasks.task_token_garbage_collector',
        'schedule': crontab(hour='*/6'),
    },
    'usage_statistics': {
        'task': 'vaultier.tasks.task_statistics_collector',
        # every day in 1 am
        'schedule': crontab(hour='1'),
    }
}



from rest_framework import serializers
from accounts.serializers import RelatedUserSerializer
from cards.models import Card
from vaultier.business.fields import PermsField


class CardSerializer(serializers.ModelSerializer):
    slug = serializers.SlugField(read_only=True)
    created_by = RelatedUserSerializer(read_only=True)
    perms = PermsField()

    class Meta:
        model = Card
        fields = ('id', 'slug', 'name', 'description', 'vault', 'perms',
                  'created_at', 'updated_at', 'created_by',)


class RelatedCardSerializer(CardSerializer):

    class Meta(CardSerializer.Meta):
        fields = ('id', 'slug', 'name')



from django.db import models
from django.db.models import CASCADE, PROTECT
from libs.tree.iterator import TreeIterableModelMixin
from libs.softdelete.softdelete import SoftDeleteMixin
from libs.changes.changes import ChangesMixin
from vaults.business.managers import VaultManager
from vaults.business.tree import VaultTreeIterator


class Vault(SoftDeleteMixin, ChangesMixin, TreeIterableModelMixin,
            models.Model):

    name = models.CharField(max_length=255)
    color = models.CharField(max_length=30, default='blue')
    slug = models.CharField(max_length=255, default='')
    description = models.CharField(max_length=1024, blank=True, null=True)
    workspace = models.ForeignKey('workspaces.Workspace', on_delete=CASCADE)
    created_by = models.ForeignKey('accounts.User', on_delete=PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    tree_iterator_class = VaultTreeIterator
    objects = VaultManager()

    class Meta:
        db_table = u'vaultier_vault'



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Version'
        db.create_table(u'vaultier_version', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('action_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True)),
            ('action_name', self.gf('django.db.models.fields.CharField')(max_length=16, null=True)),
            ('manipulator_id', self.gf('libs.version.model.VersionManipulatorIdField')(max_length=255)),
            ('revert_data', self.gf('jsonfield.fields.JSONField')()),
            ('revert_fields', self.gf('jsonfield.fields.JSONField')()),
            ('versioned_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='version_versioned', to=orm['contenttypes.ContentType'])),
            ('versioned_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('versioned_related_type', self.gf('django.db.models.fields.related.ForeignKey')(related_name='version_versioned_related', null=True, to=orm['contenttypes.ContentType'])),
            ('versioned_related_id', self.gf('django.db.models.fields.PositiveIntegerField')(null=True)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['accounts.User'], null=True, on_delete=models.PROTECT)),
        ))
        db.send_create_signal(u'versions', ['Version'])


    def backwards(self, orm):
        # Deleting model 'Version'
        db.delete_table(u'vaultier_version')


    models = {
        u'accounts.user': {
            'Meta': {'object_name': 'User', 'db_table': "u'vaultier_user'"},
            'email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'unique': 'True', 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'nickname': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'public_key': ('django.db.models.fields.CharField', [], {'max_length': '1024'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'versions.version': {
            'Meta': {'object_name': 'Version', 'db_table': "u'vaultier_version'"},
            'action_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'action_name': ('django.db.models.fields.CharField', [], {'max_length': '16', 'null': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'null': 'True', 'on_delete': 'models.PROTECT'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'manipulator_id': ('libs.version.model.VersionManipulatorIdField', [], {'max_length': '255'}),
            'revert_data': ('jsonfield.fields.JSONField', [], {}),
            'revert_fields': ('jsonfield.fields.JSONField', [], {}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'versioned_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'versioned_related_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True'}),
            'versioned_related_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'version_versioned_related'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'versioned_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'version_versioned'", 'to': u"orm['contenttypes.ContentType']"})
        }
    }

    complete_apps = ['versions']